#! /usr/bin/env python
###############################################################################
#
#  Script name                  : ChassisUpdate.py
#
#  Dependencies                 : ComDefs.py
#                    
#  Objective                    : Startup script for BladeCenterUpdates
#                    
#  Written by                   : John R. Hicks
#                                 jrhicks@us.ibm.com
#
#  Change History               : who -   when   - what
#
#                                JRH - 11/9/2004 - Initial version
#
#  Copyright (c) IBM Corporation, 2004
#
###############################################################################
try:
    import sys, getopt, string
    sys.path.append("..")
    from Common import ComDefs, ComFuncs
    from Common.MMCommunications import MMCom
    from Common.Logger import Logger
except KeyboardInterrupt:
    sys.exit(1)

# The last error that occurred
lastError = ComDefs.SUCCESS

# The MMCom class
mmCom = ""
logger = ""
params = ""
isTest = ComDefs.FALSE
standalone = ComDefs.FALSE
cleanExit = ComDefs.TRUE


class BladeServerInfo:
    isInstalled = ComDefs.FALSE
    bay = 0
    name = ""
    ipAddress = ""
    modelType = ""
    username = ""
    password = ""
    os = ""
    disabled = ComDefs.FALSE
    reboot = ComDefs.FALSE
    
    def __init__(self, bay):
        self.isInstalled = getIsBladeInstalled(bay)
        if self.isInstalled:
            self.bay = bay
            self.modelType = getBladeModelType(bay)
            self.ipAddress = ""
            self.name = ""
            self.username = ""
            self.password = ""
            self.os = ""
            self.disabled = ComDefs.FALSE
            self.reboot = ComDefs.FALSE
         
            
def getIsBladeInstalled(bladeBay=1):
    '''
        Determine if there is a Blade installed in the specified bay (1-14, 1 by
        default) of the BladeCenter chassis.
    
        Returns:
        True if the Blade is installed.
        If False is returned, you should check the global lastError variable to make
        sure it is set to ComDefs.SUCCESS.  This will ensure that there was not an
        error querying the MM and there really is not an Blade installed in the
        specified bay.
    '''
    global mmCom, lastError

    
    # Is the module installed, default to False
    isInstalled = ComDefs.FALSE
    responseByte = 0
    
    # For my own sanity...
    # The response we will get back from the MM will be 16 bytes
    # To determine if a blade is installed, we must check the bits of the 4th byte
    # for bays 1-8, and the 5th byte for bays 9-14.
    # If an blade is installed, the bit will be 1
    # If an blade is not installed, the bit will be 0
    # The bit format is:
    # Bit -      8   7   6   5  4   3   2   1
    # Value -   128 64  32  16  8   4   2   1
 
    
#    if ((bladeBay < 1) | (bladeBay > 14)):
#        lastError = ComDefs.MM_INVALID_SLOT_NUM
#        return isInstalled

    # Convert the specified bladeBay to the proper hex number so we can check the bit
    # in the response, default to 1
    slotNum = 0x01
    if bladeBay == 1:
        responseByte = 4
        pass
    elif bladeBay == 2:
        responseByte = 4
        slotNum = 0x02
    elif bladeBay == 3:
        responseByte = 4
        slotNum = 0x04
    elif bladeBay == 4:
        responseByte = 4
        slotNum = 0x08
    elif bladeBay == 5:
        responseByte = 4
        slotNum = 0x10
    elif bladeBay == 6:
        responseByte = 4
        slotNum = 0x20
    elif bladeBay == 7:
        responseByte = 4
        slotNum = 0x40
    elif bladeBay == 8:
        responseByte = 4
        slotNum = 0x80
    elif bladeBay == 9:
        responseByte = 5
        slotNum = 0x01
    elif bladeBay == 10:
        responseByte = 5
        slotNum = 0x02
    elif bladeBay == 11:
        repsonseByte = 5
        slotNum = 0x04
    elif bladeBay == 12:
        responseByte = 5
        slotNum = 0x08
    elif bladeBay == 13:
        responseByte = 5
        slotNum = 0x10
    elif bladeBay == 14:
        responseByte = 5
        slotNum = 0x20
        
    
    # Send the command and check for an error
    lastError = mmCom.sendCmd('R', '\x02\x05\x02\x01\x01', '')
    
    if lastError == ComDefs.SUCCESS:
        # Get the response and check the bits
        response = mmCom.getResponse()
        # For bays 1-8...
        # AND (&) the 4th byte of the response with the correct bit
        # If the bit is nonzero, there is a blade installed
        # otherwise, use the 5th byte.
        if (ord(response[responseByte]) & slotNum) != 0:
            isInstalled = ComDefs.TRUE
    
    return isInstalled
 
def getBladeModelType(slotNum):
    global mmCom, lastError
    
    mtm = ""
    #index = slotNum * 11
    
    lastError = mmCom.sendCmd('R', '\x1B\x01', '')
    if lastError == ComDefs.SUCCESS:
        response = mmCom.getResponse()
        
        respSplit = string.split(response, '\x00')
        i = 6
        
        while i < len(respSplit):
            #print respSplit[i]
            slot = int(respSplit[i], 16)
            if slot == slotNum:
                mtm =  respSplit[i + 4]    
        
            i = i + 12
        return mtm
        
    # return error code if dot command fails
    #print lastError
    return lastError
    
def usage():
    print "Usage: BladeInfo.py [options]"
    print "\nOptions:"
    print "  -i, --ipaddress=IPADDR   The hostname or dotted IP address of the"
    print "                           Management Module"
    print "  -u, --username=USERNAME  An administrator user name used to log into"
    print "                           the Management Module"
    print "  -p, --password=PASSWORD  The password of the administrator username"
    print "                           used to log into the Management Module"
    print "  -h, --help, -?           Display this (help) message"
    print "  -v, --verbose            Increase the debug output level by one"
    print "  -V, --version            Display the BladeInfo version number and exit"
        
def parse_args(opts, params):
    '''
       Parse the arguments and options and handle them accordingly
    '''

    global isTest, cleanExit

    for o, a in opts:
        if o in ("-?", "-h", "--help"):
            usage()
            return cleanExit
        elif o in ("-v", "--verbose"):
            params.debugLevel += 1
        elif o in ("-V", "--version"):
            print ComDefs.VERSION
            return cleanExit
        # Hidden test parameter, specify -t to use our known ip address, user name,
        # and passwords to test this script
        elif o in ("-t", "--test"):
            isTest = ComDefs.TRUE
        elif o in ("-i", "--ipaddress"):
            params.mmIPAddress = a
        elif o in ("-u", "--username"):
            params.mmUserName = a
        elif o in ("-p", "--password"):
            params.mmPassword = a
        else:
            usage()
            return cleanExit

    return ComDefs.SUCCESS


def verify_args(params):
    '''
       Verify that the required arguments have been supplied.
    '''

    global cleanExit

    if params.mmIPAddress == "":
        print "The hostname or dotted IP address of the Management Module was not specified."
        usage()
        return cleanExit

    return ComDefs.SUCCESS

def connectToMM(params):
    '''
        Connect to the MM management port (6090) and login.
    '''
    # Use global variable
    global mmCom
    return (mmCom.open(ComDefs.TRUE, params.mmIPAddress, 6090, 5, params.mmUserName, params.mmPassword))

    
def main():
    # Define using a global variable
    global mmCom, lastError, logger, isTest

    params = ComDefs.parameters()
    standalone = ComDefs.TRUE

    # Parse command line options with getopt()
    try:
        opts, args = getopt.getopt(sys.argv[1:], "?htvVi:u:p:",
             ["?", "help", "test", "verbose", "version", "ipaddress=", "username=", "password="])

    except getopt.GetoptError:
        # Print usage information and exit
        usage()
        sys.exit(0)

    lastError = parse_args(opts, params)
    if lastError != ComDefs.SUCCESS:
        return lastError

    # If we are in test mode (-t parameter), use our defaults and ignore the rest
    # of the parameters
    if isTest:
        # Populate the class
        params.mmIPAddress = ""
        params.mmUserName = ""
        params.mmPassword = ""
    else:
        lastError = verify_args(params)
        if lastError != ComDefs.SUCCESS:
            return lastError

    logger = Logger(params)
    # See if we are in debug mode
    if params.debugLevel == 0:
        # Instanciate the class and leave debug off
        mmCom = MMCom()
    else:
        # Instanciate the class and turn debug on
        mmCom = MMCom(ComDefs.TRUE)
    mmCom.logger = logger

    
    # Get the I/O module info using the 6090 management port
    # Connect to the MM port 6090
    logger.log("Retrieving Blade info from Management Module", ComDefs.TRUE)
    logger.log("Opening connection and logging into the MM")
    if connectToMM(params) != ComDefs.SUCCESS:
        logger.log("There was an error connecting or logging into the MM!", ComDefs.TRUE)
        sys.exit(1)
    else:
        logger.log("Successfully connected and logged into the MM!")

    
    for a in xrange(1,15):
        blade = BladeServerInfo(a)
        
        if blade.isInstalled == ComDefs.TRUE:
            print "Blade type %s installed in bay %d" % (getBladeModelType(a), a)
            
            
# This get's invoked if they run this script directly
if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        ComFuncs.keyboardISR(logger)
