#! /usr/bin/env python
###############################################################################
#
#  Script name          : ChassisUpdate.py
#
#  Objective            : Startup script for BladeCenterUpdates
#                    
#  Written by           : John R. Hicks <jrhicks@us.ibm.com>
#                         Robert K. Christner <christne@us.ibm.com>
#                     
#  Copyright IBM Corporation, 2004
#
###############################################################################

import sys, socket, time, getopt, string, os, urllib
try:
    from Common import ComDefs, ComFuncs
    from Common.Logger import Logger
    from Common.MMCommunications import MMCom
    from MM import MMInfo, MMFlash
    from Blades import BladeInfo
    from IOModules import *
except KeyboardInterrupt:
    sys.exit(1)
            
params = ""
logger = ""
mmCom = ""
hardwareRev = 0
numIObays = 4
numBladeBays = 14
noExtraReboot = ComDefs.FALSE

# The MM firmware packets
MM_FIRMWARE_PACKETS = {"Boot ROM":"","Main Application":"","Remote Control":""}

class outputStrings:
    # finalStatus
    Pass    = 'Success                '
    Fail    = 'Fail                   '
    Disable = 'Update Disabled        '
    Warning = 'Success (with warnings)'
    unsup   = 'Update Script Not Found'
    Linux   = 'N/A (Linux)            '
    Windows = 'N/A (Windows)          '
    bladeSpacer = '                 '
    
class ioComponentStatus:
    finalStatus = 'N/A                    '
    component   = '(empty)                               '

class bladeComponentStatus:
    finalStatus = 'N/A                    '
    component   = '(empty)                            '
    
class updateStatus:
    str = outputStrings()
    MM = ioComponentStatus()
    io = []
    # there is no I/O bay 0
    for i in xrange(numIObays + 1):
        io.append(ioComponentStatus())
    blades = []
    # there is no Blade bay 0
    for i in xrange(numBladeBays + 1):
        blades.append(bladeComponentStatus())

status = updateStatus()

def getComponentStr(id, type):

    length = 38 
    outStr = "(" + id + " " + type + ")"

    if len(outStr) < length:
        for i in xrange(length - len(outStr)):
            outStr += " "

    return outStr


def print_matrix():

    global status

    print
    print
    print "+----------------------------------------------------------------------------+"
    print "| Component                                        | Status                  |"
    print "|==================================================|=========================|"
    print "| Management Module                                | %s |" % status.MM.finalStatus 
    print "|--------------------------------------------------|-------------------------|"
    for a in xrange(numIObays):
        i = a + 1
        print "| I/O bay %s %s | %s |" % (i, status.io[i].component, status.io[i].finalStatus) 
    print "|--------------------------------------------------|-------------------------|"
    for a in xrange(numBladeBays):
        i = a + 1
        if i < 10:
            print "| Blade bay %s  %s | %s |" % (i, status.blades[i].component, status.blades[i].finalStatus)
        else:
            print "| Blade bay %s %s | %s |" % (i, status.blades[i].component, status.blades[i].finalStatus)
    print "+----------------------------------------------------------------------------+"
    print


def print_start():

    print
    print "+---------------------------------------------------------------+"
    print "| %s %s      |" % (ComDefs.PRODUCT, ComDefs.VERSION)
    print "|---------------------------------------------------------------|"
    print "| Build: %s    IBM Corporation, Copyright (c) 2004, 2005 |" % ComFuncs.getBuildID("Common/BUILDID")
    print "+===============================================================+"
    print


def usage():
    """ Displays usage instructions to the user """
    # Terminal ends ......................................................................V
    print_start()
    print "Usage: ChassisUpdate.py [options]"
    print
    print "Options:"
    print "  -r FILE, --responseFile=FILE  The path and name of the response file."
    print "  -h, --help, -?                Display this (help) message."
    print "  -v, --verbose                 Increase the debug output level by one."
    print "  -V, --version                 Display the version number and exit."
    print
    print "Examples:"
    print "  python ChassisUpdate.py --responseFile=../rsps/ux.rsp -v"
    print "  ./ChassisUpdate.py -vr ux.rsp "
    

def parse_args(opts):
    global params
    
    for o, a in opts:
        if o in ("-?", "-h", "--help"):
            usage()
            sys.exit()
        elif o in ("-v", "--verbose"):
            params.debugLevel += 1
        elif o in ("-V", "--version"):
            print_start()
            sys.exit()
        elif o in ("-r", "--responseFile"):
            params.responseFile = a 
        elif o == "mmipaddr":
            params.mmIPAddress = a
        elif o == "mmuser":
            params.mmUserName = a
        elif o == "mmpass":
            params.mmPassword = decryptPassword(a)
        elif o == "mmdisable":
            if string.upper(a) == "TRUE":
                params.mmdisable = ComDefs.TRUE
        elif o == "mmFirmwareRevision":
            pass
        elif o == "localFileLocation":
            params.localFileLocation = a
        elif o == "io1user":
            params.io1UserName = a
        elif o == "io1pass":
            params.io1Password = decryptPassword(a)
        elif o == "io1disable":
            if string.upper(a) == "TRUE":
                params.io1disable = ComDefs.TRUE
        
        ###  The FirmwareRevision and Type switch arguments are used by the
        #    BladeScanner utility, and should be ignored by ChassisUpdate.
        
        elif o == "io1FirmwareRevision":
            pass
        elif o == "io1Type":
            pass
        elif o == "io1Filename1":
            params.io1Filename1 = a
        elif o == "io1Filename2":
            params.io1Filename2 = a

        elif o == "io2user":
            params.io2UserName = a
        elif o == "io2pass":
            params.io2Password = decryptPassword(a)
        elif o == "io2disable":
            if string.upper(a) == "TRUE":
                params.io2disable = ComDefs.TRUE
        elif o == "io2FirmwareRevision":
            pass
        elif o == "io2Type":
            pass
        elif o == "io2Filename1":
            params.io2Filename1 = a
        elif o == "io2Filename2":
            params.io2Filename2 = a

        elif o == "io3user":
            params.io3UserName = a
        elif o == "io3pass":
            params.io3Password = decryptPassword(a)
        elif o == "io3disable":
            if string.upper(a) == "TRUE":
                params.io3disable = ComDefs.TRUE
        elif o == "io3FirmwareRevision":
            pass
        elif o == "io3Type":
            pass
        elif o == "io3Filename1":
            params.io3Filename1 = a
        elif o == "io3Filename2":
            params.io3Filename2 = a

        elif o == "io4user":
            params.io4UserName = a
        elif o == "io4pass":
            params.io4Password = decryptPassword(a)
        elif o == "io4disable":
            if string.upper(a) == "TRUE":
                params.io4disable = ComDefs.TRUE
        elif o == "io4FirmwareRevision":
            pass
        elif o == "io4Type":
            pass
        elif o == "io4Filename1":
            params.io4Filename1 = a
        elif o == "io4Filename2":
            params.io4Filename2 = a

        elif o == "continueIO":
            if string.upper(a) == "TRUE":
                params.continueIO = ComDefs.TRUE
        elif o == "forceMMreboot":
            if string.upper(a) == "TRUE":
                params.forceMMreboot = ComDefs.TRUE
        elif o == "UXCDLocation":
            params.UXCDLocation = a
        elif o == "ftpipaddr":
            params.ftpIPAddress = a
        elif o == "ftpuser":
            params.ftpUserName = a
        elif o == "ftppassword":
            params.ftpPassword = decryptPassword(a)
        elif o == "tftpipaddr":
            params.tftpIPAddr = a

        elif o == "blade1name":
            params.blade1name = a
        elif o == "blade1user":
            params.blade1user = a
        elif o == "blade1password":
            params.blade1pass = decryptPassword(a)
        elif o == "blade1ipaddr":
            params.blade1ipaddr = a
        elif o == "blade1os":
            params.blade1os = a
        elif o == "blade1reboot":
            if string.upper(a) == "TRUE":
                params.blade1reboot = ComDefs.TRUE
        elif o == "blade1disable":
            if string.upper(a) == "TRUE":
                params.blade1disable = ComDefs.TRUE
                
        elif o == "blade2name":
            params.blade2name = a
        elif o == "blade2user":
            params.blade2user = a
        elif o == "blade2password":
            params.blade2pass = decryptPassword(a)
        elif o == "blade2ipaddr":
            params.blade2ipaddr = a
        elif o == "blade2os":
            params.blade2os = a
        elif o == "blade2reboot":
            if string.upper(a) == "TRUE":
                params.blade2reboot = ComDefs.TRUE
        elif o == "blade2disable":
            if string.upper(a) == "TRUE":
                params.blade2disable = ComDefs.TRUE
                
        elif o == "blade3name":
            params.blade3name = a
        elif o == "blade3user":
            params.blade3user = a
        elif o == "blade3password":
            params.blade3pass = decryptPassword(a)
        elif o == "blade3ipaddr":
            params.blade3ipaddr = a
        elif o == "blade3os":
            params.blade3os = a
        elif o == "blade3reboot":
            if string.upper(a) == "TRUE":
                params.blade3reboot = ComDefs.TRUE
        elif o == "blade3disable":
            if string.upper(a) == "TRUE":
                params.blade3disable = ComDefs.TRUE
                
        elif o == "blade4name":
            params.blade4name = a
        elif o == "blade4user":
            params.blade4user = a
        elif o == "blade4password":
            params.blade4pass = decryptPassword(a)
        elif o == "blade4ipaddr":
            params.blade4ipaddr = a
        elif o == "blade4os":
            params.blade4os = a
        elif o == "blade4reboot":
            if string.upper(a) == "TRUE":
                params.blade4reboot = ComDefs.TRUE
        elif o == "blade4disable":
            if string.upper(a) == "TRUE":
                params.blade4disable = ComDefs.TRUE
                    
        elif o == "blade5name":
            params.blade5name = a
        elif o == "blade5user":
            params.blade5user = a
        elif o == "blade5password":
            params.blade5pass = decryptPassword(a)
        elif o == "blade5ipaddr":
            params.blade5ipaddr = a
        elif o == "blade5os":
            params.blade5os = a
        elif o == "blade5reboot":
            if string.upper(a) == "TRUE":
                params.blade5reboot = ComDefs.TRUE
        elif o == "blade5disable":
            if string.upper(a) == "TRUE":
                params.blade5disable = ComDefs.TRUE
                
        elif o == "blade6name":
            params.blade6name = a
        elif o == "blade6user":
            params.blade6user = a
        elif o == "blade6password":
            params.blade6pass = decryptPassword(a)
        elif o == "blade6ipaddr":
            params.blade6ipaddr = a
        elif o == "blade6os":
            params.blade6os = a
        elif o == "blade6reboot":
            if string.upper(a) == "TRUE":
                params.blade6reboot = ComDefs.TRUE
        elif o == "blade6disable":
            if string.upper(a) == "TRUE":
                params.blade6disable = ComDefs.TRUE
                
        elif o == "blade7name":
            params.blade7name = a
        elif o == "blade7user":
            params.blade7user = a
        elif o == "blade7password":
            params.blade7pass = decryptPassword(a)
        elif o == "blade7ipaddr":
            params.blade7ipaddr = a
        elif o == "blade7os":
            params.blade7os = a
        elif o == "blade7reboot":
            if string.upper(a) == "TRUE":
                params.blade7reboot = ComDefs.TRUE
        elif o == "blade7disable":
            if string.upper(a) == "TRUE":
                params.blade7disable = ComDefs.TRUE
        
        elif o == "blade8name":
            params.blade8name = a
        elif o == "blade8user":
            params.blade8user = a
        elif o == "blade8password":
            params.blade8pass = decryptPassword(a)
        elif o == "blade8ipaddr":
            params.blade8ipaddr = a
        elif o == "blade8os":
            params.blade8os = a
        elif o == "blade8reboot":
            if string.upper(a) == "TRUE":
                params.blade8reboot = ComDefs.TRUE
        elif o == "blade8disable":
            if string.upper(a) == "TRUE":
                params.blade8disable = ComDefs.TRUE
                    
        elif o == "blade9name":
            params.blade9name = a
        elif o == "blade9user":
            params.blade9user = a
        elif o == "blade9password":
            params.blade9pass = decryptPassword(a)
        elif o == "blade9ipaddr":
            params.blade9ipaddr = a
        elif o == "blade9os":
            params.blade9os = a
        elif o == "blade9reboot":
            if string.upper(a) == "TRUE":
                params.blade9reboot = ComDefs.TRUE
        elif o == "blade9disable":
            if string.upper(a) == "TRUE":
                params.blade9disable = ComDefs.TRUE
                
        elif o == "blade10name":
            params.blade10name = a
        elif o == "blade10user":
            params.blade10user = a
        elif o == "blade10password":
            params.blade10pass = decryptPassword(a)
        elif o == "blade10ipaddr":
            params.blade10ipaddr = a
        elif o == "blade10os":
            params.blade10os = a
        elif o == "blade10reboot":
            if string.upper(a) == "TRUE":
                params.blade10reboot = ComDefs.TRUE
        elif o == "blade10disable":
            if string.upper(a) == "TRUE":
                params.blade10disable = ComDefs.TRUE
                
        elif o == "blade11name":
            params.blade11name = a
        elif o == "blade11user":
            params.blade11user = a
        elif o == "blade11password":
            params.blade11pass = decryptPassword(a)
        elif o == "blade11ipaddr":
            params.blade11ipaddr = a
        elif o == "blade11os":
            params.blade11os = a
        elif o == "blade11reboot":
            if string.upper(a) == "TRUE":
                params.blade11reboot = ComDefs.TRUE
        elif o == "blade11disable":
            if string.upper(a) == "TRUE":
                params.blade11disable = ComDefs.TRUE
                
        elif o == "blade12name":
            params.blade12name = a
        elif o == "blade12user":
            params.blade12user = a
        elif o == "blade12password":
            params.blade12pass = decryptPassword(a)
        elif o == "blade12ipaddr":
            params.blade12ipaddr = a
        elif o == "blade12os":
            params.blade12os = a
        elif o == "blade12reboot":
            if string.upper(a) == "TRUE":
                params.blade12reboot = ComDefs.TRUE
        elif o == "blade12disable":
            if string.upper(a) == "TRUE":
                params.blade12disable = ComDefs.TRUE
        
        elif o == "blade13name":
            params.blade13name = a
        elif o == "blade13user":
            params.blade13user = a
        elif o == "blade13password":
            params.blade13pass = decryptPassword(a)
        elif o == "blade13ipaddr":
            params.blade13ipaddr = a
        elif o == "blade13os":
            params.blade13os = a
        elif o == "blade13reboot":
            if string.upper(a) == "TRUE":
                params.blade13reboot = ComDefs.TRUE
        elif o == "blade13disable":
            if string.upper(a) == "TRUE":
                params.blade13disable = ComDefs.TRUE
                
        elif o == "blade14name":
            params.blade14name = a
        elif o == "blade14user":
            params.blade14user = a
        elif o == "blade14password":
            params.blade14pass = decryptPassword(a)
        elif o == "blade14ipaddr":
            params.blade14ipaddr = a
        elif o == "blade14os":
            params.blade14os = a
        elif o == "blade14reboot":
            if string.upper(a) == "TRUE":
                params.blade14reboot = ComDefs.TRUE
        elif o == "blade14disable":
            if string.upper(a) == "TRUE":
                params.blade14disable = ComDefs.TRUE
                
    return ComDefs.SUCCESS
    
def validate_args():
    """ Parse provided arguments to determine if all
    	required arguments have been provided.  If not,
    	print usage message and quit.
    """
    global params
    
    if (params.mmIPAddress == ""):
        print "\nA Managament Module IP address was not specified."
        usage()
        return ComDefs.GENERAL_UPDATE_ERROR
    if (params.mmUserName == ""):
        print "\nA Managament Module username was not specified."
        usage()
        return ComDefs.GENERAL_UPDATE_ERROR

    # Validate TFTP IP address to ensure that it is both resolvable
    # and in dotted-quad notation.
    if params.tftpIPAddr != "":
        try :
            ipaddr = socket.gethostbyname(params.tftpIPAddr)
            params.tftpIPAddr = ipaddr
        except socket.gaierror:
            print "\nA valid IP address must be specified for the TFTP server."
            usage()
            return ComDefs.GENERAL_UPDATE_ERROR

    # Validate FTP IP address to ensure that it is both resolvable
    # and in dotted-quad notation.
    if params.ftpIPAddress != "":
        try:
            ipaddr = socket.gethostbyname(params.ftpIPAddress)
            params.ftpIPAddress = ipaddr
        except socket.gaierror:
            print "\nA valid IP address must be specified for the FTP server."
            usage()
            return ComDefs.GENERAL_UPDATE_ERROR
        
    return ComDefs.SUCCESS
        
def parse_responses():
    """ Read responses contained in response file, and populate
    	parameters with those responses, if they exist.
  	"""                
    # Make sure a response file was specified
    if params.responseFile == "":
        usage()
        return ComDefs.GENERAL_UPDATE_ERROR

    try:
        input = open(params.responseFile, 'r')
    except IOError:
        print "\nCannot open response file %s for reading." % params.responseFile
        return ComDefs.GENERAL_UPDATE_ERROR

    opts = []
    
    if input == None:
        print "Error! Response file is empty."
        return ComDefs.GENERAL_UPDATE_ERROR
        
    for line in input:
        line = line.strip(' ')
        if line[0] == '#' or line[0] == "\n" or line[0] == '\r':
            continue
        else:
            line = line.strip('\n')
            line = line.strip('\r')
            opt = line.split('=')
            opts.append(opt)
            
    # parse response file arguements
    rc = parse_args(opts)
    if rc != ComDefs.SUCCESS:
        return rc
    
    input.close()
    return ComDefs.SUCCESS

def decryptPassword(encPass):
    """
    	Opens a pipe to the dpw application, taking the encrypted
    	password supplied by the response file as an argument. 
    	Returns the unencrypted password
    """
    global logger
    
    fp = ""
    if os.name == "nt":
        fp = os.popen("Common\\dpw.exe \"%s\"" % encPass)
        if fp == None:
            logger.log("Unable to translate password.", ComDefs.TRUE)
            sys.exit()
    elif os.name == "posix":
        fp = os.popen("Common/dpw \"%s\"" % encPass)
        if fp == None:
            logger.log("Unable to translate password.", ComDefs.TRUE)
            sys.exit()
    else:
        logger.log("Unable to translate password, unsupported OS.", ComDefs.TRUE)
        sys.exit()
        
    password = fp.read()
    retval = fp.close()
    if retval != None:
        logger.log("Unable to translate password.", ComDefs.TRUE)
        logger.log("dpw return code: %d" % retval)
        sys.exit()

    return password
                       
    
def FlashBlades(installedBlades):
    """
        Loops through all the blade bays, calling the
        appropriate OS-specific version of RemoteUX (windows or linux)
        to update the firmware for the blade(s).
    """                    
    global logger, params, status
    
    lastError = ComDefs.SUCCESS
    
    BladeInfo.logger = logger
    BladeInfo.params = params
    
    for blade in installedBlades:
        model = blade.modelType[:4]
        status.blades[blade.bay].component = "(BladeCenter %s" % blade.modelType[:4] + ")" + status.str.bladeSpacer
            
        print
        logger.log("Processing blade bay %d" % blade.bay, ComDefs.TRUE)
                
        if blade.disabled == ComDefs.TRUE:
            print
            logger.log("Bypassing blade in bay %s (user request)" % blade.bay, ComDefs.TRUE)
            status.blades[blade.bay].finalStatus = status.str.Disable
            continue

        rebootStr = " -a -r"
        
        if blade.os == ComDefs.BLADE_OS_LINUX:
            # We should only try this block if we're running Linux,
            # so do a local OS check here.
            if os.name == "posix":
                remoteuxLoc = os.path.join(".", "remoteux")
                remoteuxStr = "%s %s -u %s -p %s -l %s -f" % (remoteuxLoc, blade.ipAddress, blade.username, blade.password, params.UXCDLocation)

                # On the JS20, the -ND (no dos) option should be appended
                # to the remoteUX command line since JS20 is not an x86-based processor.
                if model == "8842":
                    remoteuxStr += " -ND"
                else:
                    if blade.reboot == ComDefs.TRUE:
                        remoteuxStr += rebootStr 

                    
                # Print the remoteUX command line to the log for informational purposes
                logger.log("RemoteUX command line: %s" % remoteuxStr)
                
                
                # Open a pipe to the remoteUX executable and run it
                retval = os.system(remoteuxStr)                
                
                if retval != ComDefs.SUCCESS:
                    status.blades[blade.bay].finalStatus = status.str.Fail
                    logger.log("Update failed! RemoteUX returned %d while flashing blade in bay %d" % (retval, blade.bay), ComDefs.TRUE)
                else:
                    status.blades[blade.bay].finalStatus = status.str.Pass
                    logger.log("Blade Server update completed successfully.", ComDefs.TRUE)
            else:
                if blade.disabled != ComDefs.TRUE:
                    status.blades[blade.bay].finalStatus = status.str.Linux
                    logger.log("Linux blades must be updated with the Linux version of UXBC.", ComDefs.TRUE)
        elif blade.os == ComDefs.BLADE_OS_WINDOWS:
            # Same deal here, except this time check for Windows.
            if os.name == "nt":
                remoteuxLoc = os.path.join(params.UXCDLocation, "remoteux.exe")
                remoteuxStr = "%s \\\\%s -user:%s -pwd:%s -l:%s -f" % (remoteuxLoc, blade.ipAddress, blade.username, blade.password, params.UXCDLocation)
                
                if blade.reboot == ComDefs.TRUE:
                    remoteuxStr += rebootStr 

                remoteuxStr += " > NUL"

                # Print the remoteUX command line to the log for informational purposes
                logger.log("RemoteUX command line: %s" % remoteuxStr)
                
                # Open a pipe to the remoteUX executable and run it.
                retval = os.system(remoteuxStr) 
                
                if retval != ComDefs.SUCCESS:
                    status.blades[blade.bay].finalStatus = status.str.Fail
                    logger.log("Update Failed!  RemoteUX returned %d while flashing blade in bay %d" % (retval, blade.bay), ComDefs.TRUE)
                else:
                    status.blades[blade.bay].finalStatus = status.str.Pass
                    logger.log("Blade update completed successfully.", ComDefs.TRUE)
            else:
                if blade.disabled != ComDefs.TRUE:
                    status.blades[blade.bay].finalStatus = status.str.Windows
                    logger.log("Windows blades must be updated with the Windows version of UXBC.", ComDefs.TRUE)
        else:
            status.blades[blade.bay].finalStatus = status.str.Fail
            logger.log("The operating system for the blade in bay %s was improperly specified.  Update failed." % blade.bay, ComDefs.TRUE)
            
            
    
def FlashIOModules(installedModules):
    """
    	Loops through all the I/O Module bays and updates
    	the firmware for each installed module.

        if returns TN_SOCKET_ERR retry over and over and over
    """
    global logger, params, status


    # get a list of the IOModules scripts
    fileList = os.listdir("IOModules")
    classList = []
    for n in fileList:
        if os.path.splitext(n)[1] == '.py':
            classList.append(os.path.splitext(n)[0])
    try:
        classList.remove("IOInfo")
        classList.remove("__init__")
    except:
        pass

    idTable = {}

    # retrieve the manufacturer IDs from the scripts
    for n in classList:
        f = open("IOModules" + os.sep + n + ".py", 'r')
        f.readline()
        id = f.readline().strip('#').strip(os.linesep).strip(' ')
        idTable[id] = n
        f.close()

    firstpass = ComDefs.TRUE
    for module in installedModules:

        status.io[module.moduleBay].component = getComponentStr(module.manufacturer, module.type)

        if module.disable == ComDefs.TRUE:
            print
            logger.log("Bypassing %s %s in bay %s (user request)" % (module.manufacturer, module.type, module.moduleBay), ComDefs.TRUE)
            status.io[module.moduleBay].finalStatus = status.str.Disable
        else:
            if firstpass:
                maxIORetries = ComDefs.MAX_IO_RETRIES_FIRST
                firstpass = ComDefs.FALSE
            else:
                maxIORetries = ComDefs.MAX_IO_RETRIES

            if module.manufacturer in idTable:
                flashScript = globals()[idTable[module.manufacturer]]
            else:
                print
                logger.log("Bypassing %s %s in bay %s (script not found)" % (module.manufacturer, module.type, module.moduleBay), ComDefs.TRUE)
                status.io[module.moduleBay].finalStatus = status.str.unsup
                continue

            if module.manufacturer == "IBM":
                module.ipAddress = params.mmIPAddress


            flashScript.params = params
            flashScript.logger = logger
            print

            logger.log("Updating %s %s in bay %s" % (module.manufacturer, module.type, module.moduleBay), ComDefs.TRUE)
            try:
                rc = flashScript.flash_switch(module.ipAddress, module.moduleBay)
            except socket.error:
                logger.log("Error! There was a socket error while communicating with the I/O Module.")
                logger.log(ComDefs.GetLastError(ComDefs.TN_SOCKET_ERR), ComDefs.TRUE)
                continue

            if rc == ComDefs.TN_SOCKET_ERR:
                ioAttempt = 1
                while ioAttempt <= maxIORetries:
                    logger.log("Socket error connecting to %s %s (%s)" % (module.manufacturer, module.type, module.ipAddress))
                    time.sleep(10)
                    logger.log("Retry #%d of %d" % (ioAttempt, maxIORetries), ComDefs.TRUE)
                    rc = flashScript.flash_switch(module.ipAddress, module.moduleBay)
                    if rc != ComDefs.TN_SOCKET_ERR:
                        break
                    ioAttempt += 1

            if rc == ComDefs.SUCCESS:
                status.io[module.moduleBay].finalStatus = status.str.Pass
                logger.log("Success! %s %s update complete." % (module.manufacturer, module.type), ComDefs.TRUE)
            elif rc == ComDefs.WARNING:
                status.io[module.moduleBay].finalStatus = status.str.Warning
                logger.log("Success! %s %s update complete (with warnings)." % (module.manufacturer, module.type), ComDefs.TRUE)
            else:
                status.io[module.moduleBay].finalStatus = status.str.Fail
                logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
            

def FlashMM():
    """
    	Flashes the Management Module firmware by calling the MMFlash.py 
    	script.    

        Returns:
            ComDefs.SUCCESS to continue with IO
            anything else to exit
    """
    global mmCom, logger, params, status, noExtraReboot, MM_FIRMWARE_PACKETS
    
    MMInfo.logger = logger
    MMInfo.mmCom = mmCom

    rc = findPacketFiles()
    if rc != ComDefs.SUCCESS:
        logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
        status.MM.finalStatus = status.str.Fail
        return ComDefs.SUCCESS
    
    # Determine the firmware level of the update images
    packetFile = os.path.join(params.localFileLocation, MM_FIRMWARE_PACKETS["Boot ROM"])
    pktCodeLvl = getPacketRev(packetFile)
    if pktCodeLvl == 0:
        logger.log(ComDefs.GetLastError(ComDefs.MM_INVALID_PACKET_FILE), ComDefs.TRUE)
        status.MM.finalStatus = status.str.Fail
        return ComDefs.SUCCESS
    logger.log("Management Module firmware update file level: %d" % pktCodeLvl)

    # Determine the firmware level of the primary Management Module
    rc = MMInfo.getBuildIDs(1)[4:-1]
    if rc == "":
        logger.log("Error getting Management Module firmware level", ComDefs.TRUE)
        logger.log(ComDefs.GetLastError(MMInfo.lastError), ComDefs.TRUE)
        return ComDefs.GENERAL_UPDATE_FAILURE

    mmCodeLvl = int(rc)
    logger.log("Management Module firmware code level: %d" % mmCodeLvl)

    if pktCodeLvl < mmCodeLvl:
        logger.log(ComDefs.NO_BACKLEVEL, ComDefs.TRUE)
        status.MM.finalStatus = status.str.Fail
        return ComDefs.SUCCESS

    # MM code level is less than 52
    if mmCodeLvl < ComDefs.REDUNDANT_MM_CODELEVEL:
        # For now we will go ahead with this update.  If the user has redundant
        # Management Modules below 52 that are configured with DHCP, the first
        # MM will get updated, and unless the second comes up with the same IP
        # it will not be found, and the flash will fail.
        #if MMInfo.IsMMDhcp():
        #    if stopIfDhcp():
        #        return ComDefs.FALSE
        print "Updating primary Management Module."
        logger.log("Updating primary Management Module from level %d to level %d" % (mmCodeLvl, pktCodeLvl))
        rc = PerformMMFlash(ComDefs.TRUE)
        if rc == ComDefs.FALSE:
            return ComDefs.SUCCESS

        # Reconnect to the Management Module, retrying/rebooting if necessary
        rc = connectMM(ComDefs.TRUE)
        if rc != ComDefs.SUCCESS:
            logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
            status.MM.finalStatus = status.str.Fail
            return ComDefs.SUCCESS

        # query for MM firmware level
        rc = MMInfo.getBuildIDs(1)[4:-1]
        if rc == "":
            logger.log("Error getting Management Module firmware level", ComDefs.TRUE)
            logger.log(ComDefs.GetLastError(MMInfo.lastError), ComDefs.TRUE)
            return ComDefs.GENERAL_UPDATE_FAILURE
        newCodeLvl = int(rc)
        logger.log("Management Module firmware code level: %d" % newCodeLvl)

        # see if we have a redundant MM or not
        if newCodeLvl == pktCodeLvl:
            logger.log(ComDefs.MM_LEVEL_VERIFIED)
            logger.log(ComDefs.MM_FLASH_SUCCESS, ComDefs.TRUE)
            extraReboot()
            status.MM.finalStatus = status.str.Pass
            return ComDefs.SUCCESS
        elif newCodeLvl > pktCodeLvl:
            logger.log(ComDefs.NO_BACKLEVEL, ComDefs.TRUE)
            status.MM.finalStatus = status.str.Fail
            return ComDefs.SUCCESS

        # we have a redundant MM, flash it
        print "Updating redundant Management Module."
        logger.log("Updating redundant Management Module from level %d to level %d" % (newCodeLvl, pktCodeLvl))
        rc = PerformMMFlash(ComDefs.TRUE)
        if rc == ComDefs.FALSE:
            return ComDefs.SUCCESS
    # MM code level is greater than or equal to 52
    else:
        rMMBay = MMInfo.IsRedundantMMInstalled()
        # We have a redundant MM
        if rMMBay > 0:
            #logger.log("Redundant Management Module Installed", ComDefs.TRUE)
            # MM code level is less than 57
            if mmCodeLvl < ComDefs.REDUNDANT_MM_CODELEVEL:
                # The MM is configured with DHCP
                if MMInfo.IsMMDhcp():
                    if stopIfDhcp():
                        return ComDefs.GENERAL_UPDATE_ERROR
            # The MM has a static IP
            # get redundant MM firmware level
            rc = MMInfo.getRedundantBuildIDs(1)[4:-1]
            if rc == "":
                logger.log("Error getting Redundant Management Module firmware level")
                if MMInfo.lastError == ComDefs.MM_SEND_FAILURE:
                    logger.log("Assume redundant firmware level is less than 52 (i.e. 27)")
                    rc = "27"
                else:
                    logger.log(ComDefs.GetLastError(MMInfo.lastError))
                    return ComDefs.GENERAL_UPDATE_FAILURE

            redCodeLvl = int(rc)
            logger.log("Redundant Management Module firmware code level: %d" % redCodeLvl)
            # redundant is less than 57
            if redCodeLvl < ComDefs.SUPPORTED_MM_CODELEVEL:
                print "Updating primary Management Module."
                logger.log("Updating primary Management Module from level %d to level %d" % (mmCodeLvl, pktCodeLvl))
                rc = PerformMMFlash(ComDefs.TRUE)
                if rc == ComDefs.FALSE:
                    return ComDefs.SUCCESS
                # Reconnect to the Management Module, retrying/rebooting if necessary
                rc = connectMM(ComDefs.TRUE)
                if rc != ComDefs.SUCCESS:
                    logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
                    return ComDefs.SUCCESS
                rc = MMInfo.getBuildIDs(1)[4:-1]
                if rc == "":
                    logger.log("Error getting Management Module firmware level", ComDefs.TRUE)
                    logger.log(ComDefs.GetLastError(MMInfo.lastError), ComDefs.TRUE)
                    return ComDefs.GENERAL_UPDATE_FAILURE

                mmCodeLvl = int(rc)
                print "Updating redundant Management Module."
                logger.log("Updating redundant Management Module from level %d to level %d" % (mmCodeLvl, pktCodeLvl))
                rc = PerformMMFlash(ComDefs.TRUE)
                if rc == ComDefs.FALSE:
                    return ComDefs.SUCCESS
            # redundant is greater than or equal to 57
            else:
                print "Updating primary Management Module."
                logger.log("Updating primary Management Module from level %d to level %d" % (mmCodeLvl, pktCodeLvl))
                noExtraReboot = ComDefs.TRUE
                rc = PerformMMFlash()
                if rc == ComDefs.FALSE:
                    return ComDefs.SUCCESS
        # We do not have a redundant MM
        else:
            print "Updating primary Management Module."
            logger.log("Updating primary Management Module from level %d to level %d" % (mmCodeLvl, pktCodeLvl))
            rc = PerformMMFlash()
            if rc == ComDefs.FALSE:
                return ComDefs.SUCCESS

    # reboot one extra time just to be sure everything is settled
    if noExtraReboot == ComDefs.FALSE:
        extraReboot()
    # verify the firmware is at the desired level
    if verifyCodeLvl(pktCodeLvl):
        logger.log(ComDefs.MM_FLASH_SUCCESS, ComDefs.TRUE)
        status.MM.finalStatus = status.str.Pass
    else:
        status.MM.finalStatus = status.str.Fail

    return ComDefs.SUCCESS
    
def PerformMMFlash(failover=ComDefs.FALSE):
    """
        Returns:
            ComDefs.FALSE - failed
            ComDefs.TRUE - success
    """

    global status, logger, mmCom, noExtraReboot, MM_FIRMWARE_PACKETS

    MMFlash.mmCom = mmCom
    MMFlash.logger = logger


    for prettyName, packetName in MM_FIRMWARE_PACKETS.iteritems():
        packetFile = os.path.join(params.localFileLocation, packetName)
        rc = MMFlash.updateFirmware(packetFile)
        
        logger.log("MMFlash return code: %d" % rc)
        
        if rc != ComDefs.SUCCESS:
            flashSuccess = ComDefs.FALSE
            if rc == ComDefs.MM_PACKET_FILE_NOT_FOUND:
                logger.log("Flash failed:  Packet file %s not found." % packetName)
            elif ((rc == ComDefs.MM_INVALID_PACKET_FILE) |
                 (rc == ComDefs.MM_INVALID_PACKET_FILE_HEADER)):
                logger.log("\nError flashing the MM %s!\nThe packet file %s is not valid!" % (prettyName, packetName))
            else:
                logger.log("\nThere was an unknown error flashing the MM %s!" % prettyName)
        else:
            logger.log("Flashing the MM %s, please wait." % prettyName, ComDefs.TRUE)
            counter = 0
            while MMFlash.getUpdateStatus()[0] == chr(255):
                if counter % 40 == 0:
                    sys.stdout.write('.')
                    sys.stdout.flush()
                counter += 1
            print
            logger.log("Successfully flashed the MM %s." % prettyName, ComDefs.TRUE)
            flashSuccess = ComDefs.TRUE
                        
    if flashSuccess:
        logger.log("Completed MM Flash.  Rebooting MM", ComDefs.TRUE)
        logger.log("Please be patient, this can take up to 2 minutes.", ComDefs.TRUE)
        print
        MMFlash.rebootMM(failover)
        return ComDefs.TRUE
    else:
        status.MM.finalStatus = status.str.Fail
        logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
        return ComDefs.FALSE
        

def stopIfDhcp():
    """
    Returns:
        ComDefs.FALSE - if you should continue
        ComDefs.TRUE - to exit
    """
    global params, logger, status

    # If we're at code level 57 or less and using DHCP, we can't flash the 
    # redundant MM, so at this point we ask the user if they want to continue
    # flashing the I/O modules.  If the --continueIO switch is specified 
    # by the user, we bypass the prompt and just flash the switch modules.
    
    if params.continueIO == ComDefs.FALSE:
        #FIXME - pretty this up
        logger.log("Managemnet Module code level below minimum supported level for DHCP.", ComDefs.TRUE)
        logger.log("Cannot update Management Module.", ComDefs.TRUE)
        print "**To skip the prompt and automatically continue,"
        print "**please specify the continueIO key in the response file.\n"
        print "Would you like to continue flashing the I/O modules? (Y/N)"
        response = sys.stdin.read(1)
        #FIXME - do slightly better error checking here
        if string.upper(response) == "Y":
            status.MM.finalStatus = status.str.Fail
            return ComDefs.FALSE
        else:
            logger.log("User exit requested.", ComDefs.TRUE)
            return ComDefs.TRUE
    else:
        return ComDefs.FALSE

def verifyCodeLvl(pktCodeLvl):
    """
        Verifies that the current MM Code Level matches the Level of the update packet files.
        Logs onto the MM, queries, and leaves the connection open.

        Returns:
            ComDefs.TRUE - verified
            ComDefs.FALSE - unverified
    """
    global logger, mmCom

    MMInfo.logger = logger
    MMInfo.mmCom = mmCom

    logger.log("Verifying the Management Module firmware code level.", ComDefs.TRUE)
    # Reconnect to the Management Module
    rc = connectMM()
    if rc != ComDefs.SUCCESS:
        logger.log("Could not verify the firmware level of the Management Module.", ComDefs.TRUE)
        logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
        return ComDefs.FALSE

    rc = MMInfo.getBuildIDs(1)[4:-1]
    if rc == "":
        logger.log("Error getting Management Module firmware level", ComDefs.TRUE)
        logger.log(ComDefs.GetLastError(MMInfo.lastError), ComDefs.TRUE)
        return ComDefs.FALSE

    mmCodeLvl = int(rc)
    if mmCodeLvl == pktCodeLvl:
        logger.log(ComDefs.MM_LEVEL_VERIFIED, ComDefs.TRUE)
        return ComDefs.TRUE

    logger.log("The firmware level of the Management Module (%d)" % mmCodeLvl, ComDefs.TRUE)
    logger.log("is different than the update file (%d)." % pktCodeLvl, ComDefs.TRUE)
    return ComDefs.FALSE

def extraReboot(failover=ComDefs.FALSE):
    """
        Initiates a connect to the MM and reboots it via dot commands
    """
    global logger, mmCom

    MMFlash.logger = logger
    MMFlash.mmCom = mmCom
    
    logger.log("Rebooting the Management Module again to ensure all updates are settled.", ComDefs.TRUE)
    logger.log("Please be patient, this can take up to 2 minutes.", ComDefs.TRUE)
    print
    rc = connectMM()
    if rc != ComDefs.SUCCESS:
        logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
        return rc
    MMFlash.rebootMM(failover)
    return ComDefs.SUCCESS


    
def rebootMMweb(doFailOver = ComDefs.FALSE):
    """ Reboot the MM via the web interface
    Specify ComDefs.TRUE for doFailOVer if you want the MM to fail over to the redundant MM
    This is only supported on MM firmware levels at 52 or higher
    
    Return Codes:
        ComDefs.SUCCESS
        ComDefs.MM_SOCKET_ERR
    """
    global logger, params

    rc = ComDefs.SUCCESS
    if doFailOver:
        # Send the reboot command and let it fail over if there is a redundant MM
        try:
            u = urllib.urlopen("http://%s:%s@%s/private/failover_mm" % (params.mmUserName, params.mmPassword, params.mmIPAddress))
            u.close()
        except:
            logger.log("Failover of Management Module failed.  Could not connect to web interface.")
            rc = ComDefs.MM_SOCKET_ERR
    else:
        # Send the reboot command and do not let it fail over if there is a redundant MM
        try:
            u = urllib.urlopen("http://%s:%s@%s/private/restart_asm" % (params.mmUserName, params.mmPassword, params.mmIPAddress))
            u.close()
        except:
            logger.log("Restart of Management Module failed.  Could not connect to web interface.")
            rc = ComDefs.MM_SOCKET_ERR

    return rc


def handleReboot():
    """
    Reboot the Management Module of the forceMMreboot parameter is passes,
    and handle calling connectMM again if reboot succeeds.
    """
    global params, logger

    if params.forceMMreboot:
        logger.log("Rebooting the Management Module.", ComDefs.TRUE)
        logger.log("Please be patient, this can take up to 2 minutes.", ComDefs.TRUE)
        print
        rc = rebootMMweb()
        if rc != ComDefs.SUCCESS:
            logger.log("Error! Reboot of the Management Module failed.", ComDefs.TRUE)
            return ComDefs.MM_SOCKET_ERR
        else:
            #logger.log("Waiting %s seconds for the Management Module to reboot." % ComDefs.MM_REBOOT_TIME, ComDefs.TRUE)
            #time.sleep(ComDefs.MM_REBOOT_TIME)
            return connectMM()
    else:
        print
        logger.log("In this situation to force a reboot of the Management Module via", ComDefs.TRUE)
        logger.log("the web interface, please set the Force Reboot option for the MM", ComDefs.TRUE)
        logger.log("using BladeScanner.", ComDefs.TRUE)
        return ComDefs.MM_SOCKET_ERR


def connectMM(reboot=ComDefs.FALSE, maxRetries=ComDefs.MAX_MM_RETRIES):
    """
    Try to connect to the Management Module.  If there is a socket error, 
    retry "maxRetries" times.  Then reboot once, and start the retry period
    again.
    """
    global params, logger, mmCom

    logger.log("Attempting to log into Management Module.")

    rc = mmCom.open(ComDefs.TRUE, params.mmIPAddress, 6090, 5, params.mmUserName, params.mmPassword)
    
    # Because applications (like Director) access the RSA, RSA2, and
    # MM on port 6090 (which the MM uses), we loop through the MM
    # connection if we can't access port 6090.  We quit after
    # attempt number "maxRetries".
    if rc == ComDefs.MM_SOCKET_ERR:
        mmAttempt = 1
        while mmAttempt <= maxRetries:
            logger.log("Socket error connecting to Management Module (%s)" % params.mmIPAddress)
            time.sleep(10)
            logger.log("Retry #%d of %d" % (mmAttempt, maxRetries), ComDefs.TRUE)
            rc = mmCom.open(ComDefs.TRUE, params.mmIPAddress, 6090, 5, params.mmUserName, params.mmPassword)
            if rc != ComDefs.MM_SOCKET_ERR:
                break
            mmAttempt += 1
            
        if rc == ComDefs.MM_SOCKET_ERR and mmAttempt > maxRetries:
            logger.log("Maximum retries exceeded.  Unable to connect to the Management Module.", ComDefs.TRUE)
            logger.log("Verify network settings and the mmipaddr entry of the response file.", ComDefs.TRUE)
            if reboot:
                rc = handleReboot()

    # wait a few seconds before trying to access the MM
    time.sleep(5)

    return rc


def findPacketFiles():
    """
        Determines the case of the packet file names
    """
    global params, logger, MM_FIRMWARE_PACKETS

    # get a list of the files in the local directory
    fileList = os.listdir(params.localFileLocation)
    for n in fileList:
        if string.lower(n) == 'cnetbrus.pkt':
            MM_FIRMWARE_PACKETS["Boot ROM"] = n
        elif string.lower(n) == 'cnetmnus.pkt':
            MM_FIRMWARE_PACKETS["Main Application"] = n
        elif string.lower(n) == 'cnetrgus.pkt':
            MM_FIRMWARE_PACKETS["Remote Control"] = n

    if MM_FIRMWARE_PACKETS["Boot ROM"] == "":
        logger.log("Error! The Management Module Boot ROM packet file could not be found.")
        return ComDefs.MM_PACKET_FILE_NOT_FOUND
    elif MM_FIRMWARE_PACKETS["Main Application"] == "":
        logger.log("Error! The Management Module Main Application packet file could not be found.")
        return ComDefs.MM_PACKET_FILE_NOT_FOUND
    elif MM_FIRMWARE_PACKETS["Remote Control"] == "":
        logger.log("Error! The Management Module Remote Control packet file could not be found.")
        return ComDefs.MM_PACKET_FILE_NOT_FOUND

    return ComDefs.SUCCESS


def getPacketRev(packetFile):
    """
        Determines the firmware revision given the packet file
    """
    global logger

    version = 0
    
    try:
        packet = open(packetFile, 'rb')
    except IOError:
        logger.log("\nCannot open packet file %s for reading." % packetFile)
        return 0

    packet.seek(113);
    version = int(packet.read(2))
    packet.close()

    return version

def initIOModules():
    global params, logger
    
    ioModules = []
    for a in xrange(numIObays):
        switchBay = a + 1
        module = IOInfo.IOModuleInfo(switchBay)
        
        if switchBay == 1:
            if params.io1disable == ComDefs.TRUE:
                module.disable = ComDefs.TRUE
        elif switchBay == 2:
            if params.io2disable == ComDefs.TRUE:
                module.disable = ComDefs.TRUE
        elif switchBay == 3:
            if params.io3disable == ComDefs.TRUE:
                module.disable = ComDefs.TRUE
        elif switchBay == 4:
            if params.io4disable == ComDefs.TRUE:
                module.disable = ComDefs.TRUE
                
        ioModules.append(module)
        
    return ioModules

def initBlades():
    global params, logger
    
    blades = []
    for a in xrange(numBladeBays):
        bladeBay = a + 1
        blade = BladeInfo.BladeServerInfo(bladeBay)
 
        if bladeBay == 1:
            blade.ipAddress = params.blade1ipaddr
            blade.username = params.blade1user
            blade.password = params.blade1pass
            blade.name = params.blade1name
            blade.os = params.blade1os

            if params.blade1reboot == ComDefs.TRUE:
                blade.reboot = ComDefs.TRUE
            if params.blade1disable == ComDefs.TRUE:
                blade.disabled = ComDefs.TRUE
        elif bladeBay == 2:
            blade.ipAddress = params.blade2ipaddr
            blade.username = params.blade2user
            blade.password = params.blade2pass
            blade.name = params.blade2name
            blade.os = params.blade2os
            
            if params.blade2reboot == ComDefs.TRUE:
                blade.reboot = ComDefs.TRUE
            if params.blade2disable == ComDefs.TRUE:
                blade.disabled = ComDefs.TRUE
        elif bladeBay == 3:
            blade.ipAddress = params.blade3ipaddr
            blade.username = params.blade3user
            blade.password = params.blade3pass
            blade.name = params.blade3name
            blade.os = params.blade3os
            
            if params.blade3reboot == ComDefs.TRUE:
                blade.reboot = ComDefs.TRUE
            if params.blade3disable == ComDefs.TRUE:
                blade.disabled = ComDefs.TRUE
        elif bladeBay == 4:
            blade.ipAddress = params.blade4ipaddr
            blade.username = params.blade4user
            blade.password = params.blade4pass
            blade.name = params.blade4name
            blade.os = params.blade4os
            
            if params.blade4reboot == ComDefs.TRUE:
                blade.reboot = ComDefs.TRUE
            if params.blade4disable == ComDefs.TRUE:
                blade.disabled = ComDefs.TRUE
        elif bladeBay == 5:
            blade.ipAddress = params.blade5ipaddr
            blade.username = params.blade5user
            blade.password = params.blade5pass
            blade.name = params.blade5name
            blade.os = params.blade5os
            
            if params.blade5reboot == ComDefs.TRUE:
                blade.reboot = ComDefs.TRUE
            if params.blade5disable == ComDefs.TRUE:
                blade.disabled = ComDefs.TRUE
        elif bladeBay == 6:
            blade.ipAddress = params.blade6ipaddr
            blade.username = params.blade6user
            blade.password = params.blade6pass
            blade.name = params.blade6name
            blade.os = params.blade6os
            
            if params.blade6reboot == ComDefs.TRUE:
                blade.reboot = ComDefs.TRUE
            if params.blade6disable == ComDefs.TRUE:
                blade.disabled = ComDefs.TRUE
        elif bladeBay == 7:
            blade.ipAddress = params.blade7ipaddr
            blade.username = params.blade7user
            blade.password = params.blade7pass
            blade.name = params.blade7name
            blade.os = params.blade7os
            
            if params.blade7reboot == ComDefs.TRUE:
                blade.reboot = ComDefs.TRUE
            if params.blade7disable == ComDefs.TRUE:
                blade.disabled = ComDefs.TRUE
        elif bladeBay == 8:
            blade.ipAddress = params.blade8ipaddr
            blade.username = params.blade8user
            blade.password = params.blade8pass
            blade.name = params.blade8name
            blade.os = params.blade8os
            
            if params.blade8reboot == ComDefs.TRUE:
                blade.reboot = ComDefs.TRUE
            if params.blade8disable == ComDefs.TRUE:
                blade.disabled = ComDefs.TRUE
        elif bladeBay == 9:
            blade.ipAddress = params.blade9ipaddr
            blade.username = params.blade9user
            blade.password = params.blade9pass
            blade.name = params.blade9name
            blade.os = params.blade9os
            
            if params.blade9reboot == ComDefs.TRUE:
                blade.reboot = ComDefs.TRUE
            if params.blade9disable == ComDefs.TRUE:
                blade.disabled = ComDefs.TRUE
        elif bladeBay == 10:
            blade.ipAddress = params.blade10ipaddr
            blade.username = params.blade10user
            blade.password = params.blade10pass
            blade.name = params.blade10name
            blade.os = params.blade10os
            
            if params.blade10reboot == ComDefs.TRUE:
                blade.reboot = ComDefs.TRUE
            if params.blade10disable == ComDefs.TRUE:
                blade.disabled = ComDefs.TRUE
        elif bladeBay == 11:
            blade.ipAddress = params.blade11ipaddr
            blade.username = params.blade11user
            blade.password = params.blade11pass
            blade.name = params.blade11name
            blade.os = params.blade11os
            
            if params.blade11reboot == ComDefs.TRUE:
                blade.reboot = ComDefs.TRUE
            if params.blade11disable == ComDefs.TRUE:
                blade.disabled = ComDefs.TRUE
        elif bladeBay == 12:
            blade.ipAddress = params.blade12ipaddr
            blade.username = params.blade12user
            blade.password = params.blade12pass
            blade.name = params.blade12name
            blade.os = params.blade12os
            
            if params.blade12reboot == ComDefs.TRUE:
                blade.reboot = ComDefs.TRUE
            if params.blade12disable == ComDefs.TRUE:
                blade.disabled = ComDefs.TRUE
        elif bladeBay == 13:
            blade.ipAddress = params.blade13ipaddr
            blade.username = params.blade13user
            blade.password = params.blade13pass
            blade.name = params.blade13name
            blade.os = params.blade13os
            
            if params.blade13reboot == ComDefs.TRUE:
                blade.reboot = ComDefs.TRUE
            if params.blade13disable == ComDefs.TRUE:
                blade.disabled = ComDefs.TRUE
        elif bladeBay == 14:
            blade.ipAddress = params.blade14ipaddr
            blade.username = params.blade14user
            blade.password = params.blade14pass
            blade.name = params.blade14name
            blade.os = params.blade14os
            
            if params.blade14reboot == ComDefs.TRUE:
                blade.reboot = ComDefs.TRUE
            if params.blade14disable == ComDefs.TRUE:
                blade.disabled = ComDefs.TRUE
                
        
        blades.append(blade)
        
    return blades
    
def main():
    """ Main entry point """
    
    # Define using global variables
    global mmCom, params, logger, hardwareRev, status
    
    # Instanciate the class
    params = ComDefs.parameters()
 
    # Parse command line options with getopt()
    try:
        opts, args = getopt.getopt(sys.argv[1:], "?hVvr:",
            ["?", "help", "version", "verbose", "responseFile="])
    except getopt.GetoptError:
        # Print usage information and exit
        usage()
        return ComDefs.GENERAL_UPDATE_ERROR

    # parse the command line arguements
    rc = parse_args(opts)
    if rc != ComDefs.SUCCESS:
        return rc

    # parse the response file
    rc = parse_responses()
    if rc != ComDefs.SUCCESS:
        return rc

    #validate the resultant parameters
    rc = validate_args()
    if rc != ComDefs.SUCCESS:
        return rc

    # Initialize the Logger
    logger = Logger(params)

    # Print pretty start banner
    print_start()

    mmCom = MMCom(params.debugLevel)

    # Initialize dependent module logger objects
    MMInfo.logger = logger
    MMFlash.logger = logger
    MMCom.logger = logger
    mmCom.logger = logger
    BladeInfo.logger = logger
    
    # We have to pass the MMCommunications object on to
    # each of the modules we use to access the MM and I/O module.
    MMInfo.mmCom = mmCom
    MMFlash.mmCom = mmCom

    #validate the resultant parameters
    IOInfo.mmCom = mmCom
    BladeInfo.mmCom = mmCom

    logger.log("Connecting to the BladeCenter Management Module.", ComDefs.TRUE)
    
    # Connect to the Management Module, retrying/rebooting if necessary
    rc = connectMM(ComDefs.TRUE, 3)
    if rc != ComDefs.SUCCESS:
        logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
        return rc

    logger.log("Successfully logged into management module.")
        
    hardwareRev = MMInfo.getChassisHardwareRev()
    if hardwareRev == -1:
        logger.log("Error getting hardware revision.", ComDefs.TRUE)
        logger.log(ComDefs.GetLastError(MMInfo.lastError), ComDefs.TRUE)
        return ComDefs.GENERAL_UPDATE_ERROR

    logger.log("Chassis hardware revision: %d" % hardwareRev)
        
    # If the hardware revision of the chassis midplane is less than 5,
    # display an error and exit.
    
    # JRH - Check to see if we have a Blackbeard Chassis, which is
    # a scaled-down version of the BladeCenter chassis.  The way to 
    # determine this is to examine the number of I/O or blade bays.  
    # If we have fewer than the number in a normal BC Chassis (4 I/O bays, 14 Blade bays),
    # then we have a BlackBeard, and we must look for a different midplane VPD level.
    
    chassisType = MMInfo.getChassisModelType()
    logger.log("BladeCenter Chassis Type: %s" % chassisType)
    
    if chassisType == "8677":
        if (hardwareRev < ComDefs.SUPPORTED_MIDPLANE_LEVEL):
            message = "Hardware revision %d is not supported.  Must be at least revision %s." \
                       % (hardwareRev, ComDefs.SUPPORTED_MIDPLANE_LEVEL)
            logger.log(message, ComDefs.TRUE)
            return ComDefs.GENERAL_UPDATE_ERROR
    
    
    
    # Loop through all the I/O modules and gather their info.  We do this so that we
    # ensure that we can flash the I/O modules after rebooting the MM.
    
    # Create IOInfo.IOModuleInfo objects for each of the 4 switch bays.
    ioModules = initIOModules()
    
    installedModules = []
    for a in ioModules:
        if a.isInstalled == ComDefs.TRUE:
            #status.io[a.moduleBay].finalStatus = status.str.unsup
            installedModules.append(a)

    blades = initBlades()
    
    installedBlades = []
    for a in blades:
        if a.bay < 10:
            status.blades[a.bay].component += ' '
        if a.isInstalled == ComDefs.TRUE:
            installedBlades.append(a)
            
    # Flash the MM
    print "\nUpdating Management Module"
    print "--------------------------\n"
 
    if params.mmdisable != ComDefs.TRUE:
        rc = FlashMM()
        if rc != ComDefs.SUCCESS:
            return rc
    else:
        status.MM.finalStatus = status.str.Disable
        logger.log("Bypassing Management Module Update (user request)", ComDefs.TRUE)
    
    # Loop through all the I/O modules, gather their info, and flash them.
    print "\n\nUpdating I/O Module(s)"
    print "----------------------"
    FlashIOModules(installedModules)

    print "\n\nUpdating Blade(s)"
    print "-----------------"
    FlashBlades(installedBlades)

    mmCom.close()

    print_matrix()
    logger.close()

# This get's invoked if they run this script directly
if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        ComFuncs.keyboardISR(logger)
