#! /usr/bin/env python
###############################################################################
#
#  Script name                  : MMComunications.py
#
#  Dependencies                 : ComDefs.py
#                    
#  Objective                    : Communication class for connecting to the management
#                                 port (6090) of the Management Module and running dot
#                                 commands
#                    
#  Written by                   : Mike Nolterieke
#                                 mikenol@us.ibm.com
#                     
#  Change History               : who -   when   - what
#                                 MHN - 07/01/04 - Initial revision.
#                                 RKC - 08/01/04 - added logging
#  
#  Copyright IBM Corporation, 2004
#
###############################################################################
""" Communication class for connecting to the management port (6090) of the
    Management Module and running dot commands
"""
try:
    import socket, sys, time
    sys.path.append("..")
    import ComDefs
except KeyboardInterrupt:
    sys.exit(1)

logger = ""

class MMCom:
    
    def __init__(self, isDebug=ComDefs.FALSE):
        """ Constructor that sets the optional global debug flag
        Specify True (default is False) for isDebug to display addtional
        messages when using the functions of this class
        """
        self.logger = logger
        self.isDebug = isDebug

    def open(self, doConnect, host, port=6090, timeOut=5, userName="USERID", password="PASSW0RD"):
        """ Configure and open the connection to the MM specified by the host parameter
        using the specified port (6090 by default) and the specified timeout value
        (5 seconds by default)
        
        If doConnect is True, this function will call the sendUserName function using the
        specified userName (USERID by default) and the sendPassword function using the
        specified password (PASSW0RD by default) to login into the specified MM
         
        Return Codes:
            ComDefs.SUCCESS
            ComDefs.MM_SOCKET_ERR
            ComDefs.MM_BAD_IP
            ComDefs.MM_UNKNOWN_ERR
            ComDefs.MM_SOCKET_TIMEOUT
            Any return codes possible from the sendUserName and sendPassword functions
        """

        # Check to verify that the username is between 1 and 15 characters
        if len(userName) < 1 or len(userName) > 15:
            self.logger.log("open -- Username is invalid.  Usernames must contain between 1 and 15 characters.")
            return ComDefs.MM_INVALID_LOGIN
            
        # Check to verify that the password is between 5 and 15 characters if a password
        # is specified at all.  The MM spec allows for zero-length passwords.
        if len(password) > 0:
            if len(password) < 5 or len(password) > 15:
                self.logger.log("open -- Password is invalid.  Empty passwords are permitted, but if a password is specified, it must contain between 5 and 15 characters.")
                return ComDefs.MM_INVALID_LOGIN

        self.host = host
        self.port = port

        self.logger.log("open -- Opening connection to MM %s on port %d." % (self.host, self.port))

        # Create the socket and handle any errors
        try:
            # Configure the socket
            self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        except:
            self.logger.log("open -- Unknown socket error %s\nData was: %s" % (sys.exc_info()[0], sys.exc_info()[1]))
            return ComDefs.MM_SOCKET_ERR

        # Connect to the MM and handle any errors
        try:
            self.sock.connect((self.host, self.port))
        except socket.gaierror:
            self.logger.log("open -- Bad IP error")
            return ComDefs.MM_BAD_IP
        except socket.error:
            self.logger.log("open -- Socket error")
            return ComDefs.MM_SOCKET_ERR
        except socket.timeout:
            self.logger.log("open -- Socket connect timout.")
            return ComDefs.MM_SOCKET_TIMEOUT
        except:
            self.logger.log("open -- Unknown connect error %s\nData was: %s" % (sys.exc_info()[0], sys.exc_info()[1]))
            return ComDefs.MM_UNKNOWN_ERR

        # If they do not want to login yet, return success
        if doConnect == ComDefs.FALSE:
            return ComDefs.SUCCESS

        # If they want to login...
        # Send the specified user name and get the encryption keys back
        keys = self.sendUserName(userName)
        # Make sure we got the encryption keys
        if keys == ComDefs.MM_INVALID_KEYS:
            return keys
        # Send the specified password using the encryption keys
        rc = self.sendPassword(password, keys);
        # Return the return code from sendPassword
        return rc

    def close(self):
        """ Closes the communication socket"""
        self.logger.log("close -- Closing communications socket...")
        self.sock.close()
        self.sock = 0

    def sendUserName(self, userName):
        """ Send the specified user name to log into the MM and receive and return the encryption keys
        Return Codes:
            ComDefs.MM_INVALID_KEYS"""
        self.logger.log("sendUserName -- Sending username to MM...")
        self.sock.send('SPREMOTE'+ userName + chr(0))

        self.logger.log("sendUserName -- Receiving encryption keys...")
        keys = self.sock.recv( 40 )
        if len(keys) != 2:
            self.logger.log("sendUserName -- Encryption keys not received")
            return ComDefs.MM_INVALID_KEYS
        self.logger.log("sendUserName -- Encryption keys received")
        return keys

    def sendPassword(self, password, keys):
        """ Send the encrypted password to the MM using the specified password and encryption keys 
        Return Codes: 
            ComDefs.SUCCESS
            ComDefs.MM_INVALID_LOGIN
            ComDefs.MM_RETRY_LIMIT_EXCEEDED
        """
        keybyte0 = (ord(keys[0]))
        keybyte1 = (ord(keys[1]))

        # pw = password exclusive OR'd with 1st keybyte then added with 2nd keybyte
        pw = ''
        i = 0
        while i < len(password):
            pw += chr(((ord(password[i]) ^ keybyte0) + keybyte1) & 0xFF)
            i += 1

        # Send the encrypted password
        self.logger.log("sendPassword -- Sending encrypted password to MM...")
        self.sock.send(pw + chr(0))

        # Receive the acknowledgement back from the login attempt and check for errors
        ack = self.sock.recv(80)
        if ack == "BADLOGIN\0":
            self.logger.log("sendPassword -- A invalid user ID or password was specified!")
            return ComDefs.MM_INVALID_LOGIN
        elif ack == "RETRYLIMIT\0":
            self.logger.log("sendPassword -- The retry limit has been exceeded!")
            return ComDefs.MM_RETRY_LIMIT_EXCEEDED
        else:
            self.logger.log("sendPassword -- Login was successful!")
            return ComDefs.SUCCESS

        return ComDefs.SUCCESS

    def sendCmd(self, cmdType, cmd, data):
        """ Sends the specified dot cmd and data to the opened connection, waits
        for response, and validates that the received response is for the issued cmd.
        The response data can then be obtained via the getResponse() method.

        The command type (cmdType) must be one of the following:
            R   = Read
            RN  = Read Next
            W   = Write
            WN  = Write Next

        The command (cmd) must be in hex format (\\x07\\x04 to reboot the MM)

        Return Codes:
            ComDefs.SUCCESS
            ComDefs.MM_UNKNOWN_CMD_TYPE
            ComDefs.MM_UNKNOWN_ERR
            ComDefs.MM_INVALID_RESPONSE
            ComDefs.MM_LENGTH_MISMATCH
            ComDefs.MM_COMMAND_MISMATCH
            ComDefs.MM_SEND_FAILURE
        """
        
        # Get the length of the command
        cmdLength = len(cmd)
        # Keep the length as a global variable
        self.cmdLen = cmdLength
        # Determine the type of command and convert it to an int
        # Write
        if cmdType == 'W':
            cmdType = 0
        # Write Next
        elif cmdType == 'WN':
            cmdType = 1
        # Read
        elif cmdType == 'R':
            cmdType = 2
        # Read Next
        elif cmdType == 'RN':
            cmdType = 3
        # Unknown
        else:
            self.logger.log("sendCmd -- Invalid command type")
            return ComDefs.MM_UNKNOWN_CMD_TYPE
        # Convert the command type to hex
        cmdType = chr(cmdType)

        count = 0
        # We will retry the command we succeed or reach MAX_MM_SEND_RETRIES
        while count < ComDefs.MAX_MM_SEND_RETRIES:
            # Send the command
            try:
                self.sock.send(cmdType + chr(cmdLength) + chr(len(data)) + (chr(0)*3) + cmd + data + chr(0))
            except:
                self.logger.log("sendCmd -- Socket send() exception ignored.")

            # Skip the response data when restarting
            if ((cmd == '\x07\x04') | (cmd == '\x07\x04\x01')):
                self.logger.log("sendCmd -- Reboot command was issued, skipping response.")
                return ComDefs.SUCCESS

            # Get the response
            try:
                self.response = self.sock.recv(1500)
            # Handle errors
            except socket.timeout:
                self.logger.log("sendCmd -- Socket recv() timout.")
            except socket.error:
                self.logger.log("sendCmd -- Socket error")
                return ComDefs.MM_SOCKET_ERR
            except:
                self.logger.log("sendCmd -- Unknown error %s getting response\nData was: %s" % (sys.exc_info()[0], sys.exc_info()[1]))
                return ComDefs.MM_UNKNOWN_ERR

            # Validate the response
            # Invalid responses
            if (len(self.response) == 0):
                self.logger.log("sendPacket -- Response is empty")
                #return ComDefs.MM_INVALID_RESPONSE
            if self.response[0] != chr(4):
                self.logger.log("sendCmd -- Response %s is not valid" % ord(self.response[0]))
                #return ComDefs.MM_INVALID_RESPONSE
            # Length mismatch
            if self.response[1] != chr(cmdLength):
                self.logger.log("sendCmd -- Response length %s does not match %s" % (ord(self.response[1]), chr(cmdLength)))
                #return ComDefs.MM_LENGTH_MISMATCH
            # Command mismatch
            if (self.response[6:6+cmdLength] != cmd[0:cmdLength]):
                self.logger.log("sendCmd -- Response command %s does not match %s" % (self.response[6:6+cmdLength], cmd[0:cmdLength]))
                #return ComDefs.MM_COMMAND_MISMATCH

            self.rtnstatus = ord(self.response[4])
            if self.rtnstatus == 0:
                break
            #if ((self.rtnstatus != 6) & (self.rtnstatus != 7)):
            #    break
                    
            count += 1

            if count < ComDefs.MAX_MM_SEND_RETRIES:
                self.logger.log("sendCmd -- Retrying SP command #%d of %d" % (count, ComDefs.MAX_MM_SEND_RETRIES))
                self.logger.log("sendCmd -- Response status was %d - '%s'" % (self.rtnstatus, ComDefs.MM_CMD_STATUS[self.rtnstatus]))
                # Sleep for 1 second and then retry
                time.sleep(1)
            else:
                return ComDefs.MM_SEND_FAILURE
        
        # If we tried the command too many times and did not succeed
        if self.rtnstatus != 0:
            self.logger.log("sendCmd -- Send failure!")
            #self.logger.log("sendCmd -- Response status was %d - '%s'" % (self.rtnstatus, ComDefs.MM_CMD_STATUS[self.rtnstatus]))
            return ComDefs.MM_SEND_FAILURE

        # Print out command and response in hex if we are in debug mode
        #if self.isDebug:
        if ComDefs.FALSE:
            # Convert the response
            self.datalen  = ord(self.response[3]) * 256
            self.datalen += ord(self.response[2])
            self.datahex = ''
            indx = 0
            indx2 = 6 + self.cmdLen
            while indx < self.datalen:
                if indx2 >= len(self.response):
                    break
                self.datahex += "%02X" % ord(self.response[indx2])
                indx += 1
                indx2 += 1
                # Only add the comma if it is not the last one
                if indx < self.datalen:
                    self.datahex += ","

            # Convert the command
            self.cmdHex = ''
            indx = 0
            while indx < len(cmd):
                self.cmdHex += "%X" % ord(cmd[indx])
                indx += 1
                # Only add the period if it is not the last one
                if indx < len(cmd):
                    self.cmdHex += "."

            self.logger.log('sendCmd -- Hex response data for dot command %s is %s' % (self.cmdHex, self.datahex))

        # Get the length of the response
        indx2 = 6 + self.cmdLen
        # Put the response in a global variable
        self.responseData = self.response[indx2:]
        return ComDefs.SUCCESS

    def getResponse(self):
        """  Returns the last dot command response data.
        """
        return self.responseData

    def sendPacket(self, packet):
        """ Sends the specified packet file to the opened connection, waits
        for response, and validates that the received response is for the issued cmd.

        Return Codes:
            ComDefs.SUCCESS
            ComDefs.MM_INVALID_RESPONSE
            ComDefs.MM_LENGTH_MISMATCH
            ComDefs.MM_COMMAND_MISMATCH
            ComDefs.MM_SEND_FAILURE
        """
        try:
            self.sock.send(packet)
        except socket.timeout:
            self.logger.log("sendCmd -- Socket send() timout.")
            return ComDefs.MM_SOCKET_TIMEOUT
        except:
            self.logger.log("sendPacket -- Unknown error %s sending the packet\nData was: %s" % (sys.exc_info()[0], sys.exc_info()[1]))
            return ComDefs.MM_UNKNOWN_ERR
        
        # Get the response
        try:
            self.response = self.sock.recv(200)
        # Handle errors
        except socket.timeout:
            self.logger.log("sendCmd -- Socket recv() timout.")
            return ComDefs.MM_SOCKET_TIMEOUT
        except:
            self.logger.log("sendPacket -- Unknown error %s getting response\nData was: %s" % (sys.exc_info()[0], sys.exc_info()[1]))
            return ComDefs.MM_UNKNOWN_ERR

        self.cmdType = packet[0]
        self.cmdLen  = ord(packet[1])

        # Validate the response
        # Invalid responses
        # There is a problem when using the VPN dialer where the response
        # comes back empty
        if (len(self.response) == 0):
            self.logger.log("sendPacket -- Response is empty")
            return ComDefs.MM_INVALID_RESPONSE
        if self.response[0] != '\x04':
            #self.logger.log("sendPacket -- Response %s is not valid" % ord(self.response[0]))
            self.logger.log("sendPacket -- Response is not valid")
            return ComDefs.MM_INVALID_RESPONSE
        # Length mismatch
        if ord(self.response[1]) != self.cmdLen:
            #self.logger.log("sendPacket -- Response length %s does not match %s" % (ord(self.response[1]), chr(cmdLen)))
            self.logger.log("sendPacket -- Response length does not match")
            return ComDefs.MM_LENGTH_MISMATCH
        # Command mismatch
        if (packet[6:6 + self.cmdLen] != self.response[6:6 + self.cmdLen]):
            #self.logger.log("sendPacket -- Response command %s does not match %s" % (self.response[6:6+cmdLength], cmd[0:cmdLength]))
            self.logger.log("sendPacket -- Response command does not match")
            return ComDefs.MM_COMMAND_MISMATCH

        # Send failure
        self.rtnstatus = ord(self.response[4])
        if self.rtnstatus != 0:
            self.logger.log("sendPacket -- Send failure!")
            #self.logger.log("sendPacket -- Response status was %d - '%s'" % (self.rtnstatus, ComDefs.MM_CMD_STATUS[self.rtnstatus]))
            return ComDefs.MM_SEND_FAILURE

        # Return success
        return ComDefs.SUCCESS

