#! /usr/bin/env python
#BRCD 
###############################################################################
#
#  Script name                  : BrocadeSMFlash.py
#
#  Dependencies                 : ComDefs.py
#                                 ComFuncs.py
#                                 Logger.py
#                    
#  Objective                    : Script used to login to a Brocade switch
#                                 using telnet and update the firmware via an
#                                 FTP server.
#                    
#  Written by                   : Robert K. Christner
#                                 christne@us.ibm.com
#                     
#  Change History               : who -   when   - what
#                                 RKC - 10/07/04 - Initial revision.
#  
#  Copyright IBM Corporation, 2004
#
###############################################################################
"""
   Script used to login to a Brocade switch over telnet and update the
   firmware via a FTP server.
"""
try:
    import time, sys, os, re, getopt, telnetlib, socket
    sys.path.append("..")
    from Common import ComDefs, ComFuncs
    from Common.Logger import Logger
except KeyboardInterrupt:
    sys.exit(1)


# Instantiate Globals
params = ""
logger = ""
lastError = ComDefs.SUCCESS
ipAddr = ComDefs.FALSE
cleanExit = ComDefs.TRUE
standalone = ComDefs.FALSE

def usage():
    '''
        Display the Usage Information to the screen
    '''

    print "Usage: BrocadeSMFlash.py [options]"
    print "\nOptions:"
    print "  -i, --ipaddress=IPADDR      The hostname or dotted IP address of the I/O"
    print "                              Module"
    print "  -u, --username=USERNAME     An administrator user name used to log into the"
    print "                              I/O Module"
    print "  -p, --password=PASSWORD     The password of the administrator user name used"
    print "                              to log into the I/O Module"
    print "  -s, --ftpserver=IPADDR      The dotted IP address of the FTP server hosting"
    print "                              the new firmware image"
    print "  -m, --ftpusername=USERNAME  An user name used to log into the FTP server"
    print "  -w, --ftppassword=PASSWORD  The password of the user name used to"
    print "                              log into the FTP server"
    print "  -n, --imagename=FILE        The path and name of the I/O Module firmware image"
    print "                              file on the FTP server"
    print "  -h, --help, -?              Display this (help) message"
    print "  -v, --verbose               Increase the debug output level by one"
    print "  -V, --version               Display the BrocadeSMFlash version number and exit"


def parse_args(opts):
    """
       Parse the arguments and options and handle them accordingly
    """

    global params, ipAddr, cleanExit

    for o, a in opts:
        if o in ("-?", "-h", "--help"):
            usage()
            return cleanExit
        elif o in ("-v", "--verbose"):
            params.debugLevel += 1
        elif o in ("-V", "--version"):
            print ComDefs.VERSION
            return cleanExit
        elif o in ("-i", "--ipaddress"):
            ipAddr = a
        elif o in ("-u", "--username"):
            params.io1UserName = a
        elif o in ("-p", "--password"):
            params.io1Password = a
        elif o in ("-s", "--ftpserver"):
            params.ftpIPAddress = a
        elif o in ("-m", "--ftpusername"):
            params.ftpUserName = a
        elif o in ("-w", "--ftppassword"):
            params.ftpPassword = a
        elif o in ("-n", "--imagename"):
            params.io1Filename1 = a
        else:
            usage()
            return cleanExit

    return ComDefs.SUCCESS

def verify_args():
    """
       Verify that the required arguments have been supplied.
    """

    global params, ipAddr, cleanExit

    if ipAddr == "":
        print "The hostname or dotted IP address of the I/O Module was not specified."
        usage()
        return cleanExit
    if params.ftpIPAddress == "":
        print "The dotted IP address of the FTP server was not specified."
        usage()
        return cleanExit
    if params.io1Filename1 == "":
        print "The path and name of the I/O Module firmware image file on the FTP"
        print "server was not specified."
        usage()
        return cleanExit

    return ComDefs.SUCCESS


def connect(telnet, ioIPAddr, userName, password):
    """
        Connect and authenticate to the switch.
    """

    global params, logger, standalone

    try:
        telnet.open(ioIPAddr)
        logger.log("Connecting to %s" % ioIPAddr)
    except socket.error:
        logger.log("Socket error! Connection refused or timed out to (%s)." % ioIPAddr)
        lastError = ComDefs.TN_SOCKET_ERR
        return lastError
    except KeyboardInterrupt:
        sys.exit(0)

    # look for "login" and enter credentials
    reList = []
    reList.append(re.compile('login'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Error! Telnet timeout.")
        if params.debugLevel > 1:
            print txt
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Connected to Fibre Channel Switch Module (%s)." % ioIPAddr)
    telnet.write('%s\r' % userName)
    logger.log("Sending username.")
    # look for "Password" and enter credentials
    reList = []
    reList.append(re.compile('Password'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Error! Telnet timeout.")
        if params.debugLevel > 1:
            print txt
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    telnet.write('%s\r' % password)
    logger.log("Sending password.")
    #logger.log("Sending password (%s)." % password)
    # credentials are sent, continue to the next screen
    reList = []
    reList.append(re.compile('>'))
    reList.append(re.compile('incorrect'))
    reList.append(re.compile('change\ your\ passwords\ now'))
    (index,mo,txt) = telnet.expect(reList)
    if index == 1:
        logger.log("Error! Username or Password not accepted.")
        if params.debugLevel > 1:
            print txt
        telnet.close()
        lastError = ComDefs.TN_INVALID_LOGIN
        return lastError
    if index == 2:
        # asks to change password
        # just hit Ctrl-C to continue
        telnet.write('\r')
    logger.log("Username and Password accepted.")

    lastError = ComDefs.SUCCESS
    return lastError


def update_firmware(telnet, ioIPAddr, userName, password, filename):
    """
        Flash the firmware
    """

    global params, logger, standalone

    # open a telnet session with the switch
    lastError = connect(telnet, ioIPAddr, userName, password)
    if lastError != ComDefs.SUCCESS:
        return lastError

    # credentials are accepted, continue to firmware upgrade
    telnet.write('firmwaredownload\r')
    reList = []
    reList.append(re.compile('continue'))
    reList.append(re.compile('command\ not\ found'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Error! Telnet timeout.")
        if params.debugLevel > 1:
            print txt
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    elif index == 2:
        logger.log("Error! firmware update command not found.")
        if params.debugLevel > 1:
            print txt
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    telnet.write('y\r')

    # answer prompts
    # Server Name or IP Address:
    # FIXME - may ask for this twice if a hostname is entered
    reList = []
    reList.append(re.compile('Address'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Error! Telnet timeout.")
        if params.debugLevel > 1:
            print txt
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Entering FTP server IP address (%s)." % params.ftpIPAddress)
    telnet.write('%s\r' % params.ftpIPAddress)
    # User Name:
    reList = []
    reList.append(re.compile('User'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Error! Telnet timeout.")
        if params.debugLevel > 1:
            print txt
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Entering User Name.")
    telnet.write('%s\r' % params.ftpUserName)
    # File Name:
    reList = []
    reList.append(re.compile('File'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Error! Telnet timeout.")
        if params.debugLevel > 1:
            print txt
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Entering firmware update file name (%s)." % filename) 
    telnet.write('%s\r' % filename)
    # Password:
    reList = []
    reList.append(re.compile('Password'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Error! Telnet timeout.")
        if params.debugLevel > 1:
            print txt
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Entering password.")
    telnet.write('%s\r' % params.ftpPassword)

    # downloading firmware
    reList = []
    reList.append(re.compile('install\ packages'))
    reList.append(re.compile('annot\ download'))
    reList.append(re.compile('already\ in\ progress'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Error! Telnet timeout.")
        if params.debugLevel > 1:
            print txt
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    elif index == 1:
        logger.log("Error! firmware download failed.")
        if params.debugLevel > 1:
            print txt
        telnet.close()
        lastError = ComDefs.IO_FIRMWARE_IMAGE_NOT_FOUND
        return lastError
    elif index == 2:
        logger.log("Error! firmware download already in progress.")
        if params.debugLevel > 1:
            print txt
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Downloading Fibre Channel Switch Module firmware image from FTP server.", ComDefs.TRUE)
    # wait for download to complete or an error to occur
    reList = []
    reList.append(re.compile('flash'))
    reList.append(re.compile('\r'))
    while ComDefs.TRUE:
        (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
        if index == -1:
            print
            logger.log("Error! Telnet timeout.")
            if params.debugLevel > 1:
                print txt
            telnet.close()
            lastError = ComDefs.GENERAL_UPDATE_ERROR
            return lastError
        elif index == 0:
            print
            logger.log("Writing kernel image into FLASH", ComDefs.TRUE)
            break
        elif index == 1:
            sys.stdout.write('.')
            sys.stdout.flush()

    # Downloading is complete, now writing
    reList = []
    reList.append(re.compile('SUCCEEDED'))
    reList.append(re.compile('\.\.'))
    while ComDefs.TRUE:
        (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
        if index == -1:
            print
            logger.log("Error! Write failed")
            if params.debugLevel > 1:
                print txt
            telnet.close()
            lastError = ComDefs.GENERAL_UPDATE_ERROR
            return lastError
        elif index == 0:
            print
            logger.log("Image has been verified.")
            break
        elif index == 1:
            sys.stdout.write('.')
            sys.stdout.flush()

    # Writing is complete, now wait for update to finish
    reList = []
    reList.append(re.compile('successfully'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Error! Update failed")
        if params.debugLevel > 1:
            print txt
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError

    # wait for reboot
    reList = []
    reList.append(re.compile('Rebooting'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Error! Update failed")
        if params.debugLevel > 1:
            print txt
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError

    if standalone:
        logger.log("Fibre Channel Switch Module firmware succesfully updated.", ComDefs.TRUE)

    telnet.close()
    lastError = ComDefs.SUCCESS
    return lastError


def flash_switch(ioIPAddr, ioBay=1):
    """
       Flash the Brocade switch via interaction with the telnet menu interface.
    """

    global params, logger, standalone, lastError

    if standalone:
        logger.log("Updating Fibre Channel Switch Module firmware.", ComDefs.TRUE)

    # Get Variables from params class
    if ioBay == 1:
        userName = params.io1UserName
        password = params.io1Password
        filename = params.io1Filename1
    elif ioBay == 2:
        userName = params.io2UserName
        password = params.io2Password
        filename = params.io2Filename1
    elif ioBay == 3:
        userName = params.io3UserName
        password = params.io3Password
        filename = params.io3Filename1
    elif ioBay == 4:
        userName = params.io4UserName
        password = params.io4Password
        filename = params.io4Filename1

    if params.ftpIPAddress == "":
        logger.log("Error! The IP Address of the ftp server was not specified.")
        lastError = ComDefs.FTP_BAD_IP
        return lastError
    if params.ftpUserName == "":
        logger.log("Error! The Username for the ftp server was not specified.")
        lastError = ComDefs.FTP_INVALID_LOGIN
        return lastError
    if userName == "":
        logger.log("Error! Username was not supplied.")
        lastError = ComDefs.TN_INVALID_LOGIN
        return lastError
    if password == "":
        logger.log("Error! Password was not supplied.")
        lastError = ComDefs.TN_INVALID_LOGIN
        return lastError
    if filename == "":
        logger.log("Error! Firmware image filename was not supplied.")
        lastError = ComDefs.IO_INVALID_IMAGE_fILE
        return lastError

    telnet = telnetlib.Telnet()

    # flash the firmware
    lastError = update_firmware(telnet, ioIPAddr, userName, password, filename)
    if lastError != ComDefs.SUCCESS:
        return lastError

    if standalone:
        logger.log("Success! Fibre Channel Switch Module firmware update complete.", ComDefs.TRUE)

    return ComDefs.SUCCESS


def main():
    """ Main function used to run this script in standalone mode"""

    # Global Variables
    global params, logger, ipAddr, standalone, lastError

    params = ComDefs.parameters()
    standalone = ComDefs.TRUE

    # Parse command line options with getopt()
    try:
        opts, args = getopt.getopt(sys.argv[1:], "?hvVi:u:p:s:m:w:n:",
             ["?", "help", "verbose", "version", 
              "ipaddress=", "username=", "password=",
              "ftpserver=", "ftpusername=", "ftppassword=", "imagename="])

    except getopt.GetoptError:
        # Print usage information and exit
        usage()
        sys.exit(0)
    
    lastError = parse_args(opts)
    if lastError != ComDefs.SUCCESS:
        return lastError

    lastError = verify_args()
    if lastError != ComDefs.SUCCESS:
        return lastError
        
    logger = Logger(params)
    rc = flash_switch(ipAddr)
    if rc != ComDefs.SUCCESS:
        logger.log(ComDefs.GetLastError(rc), ComDefs.TRUE)
    
# This get's invoked if they run this script directly
if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        ComFuncs.keyboardISR(logger)
