#! /usr/bin/env python
#CSCO
###############################################################################
#
#  Script name                  : CiscoSMFlash.py
#
#  Dependencies                 : ComDefs.py
#                                 ComFuncs.py
#                                 Logger.py
#                    
#  Objective                    : Script used to login to a Cisco switch 
#                                 using telnet and update the firmware via a 
#                                 TFTP server.
#                    
#  Written by                   : Robert K. Christner
#                                 christne@us.ibm.com
#                     
#  Change History               : who -   when   - what
#                                 RKC - 08/23/04 - Initial revision.
#  
#  Copyright IBM Corporation, 2004
#
###############################################################################
"""
   Script used to login to a Cisco switch over telnet and update the
   firmware via a TFTP server.
"""
try:
    import time, sys, os, re, getopt, telnetlib, socket
    sys.path.append("..")
    from Common import ComDefs, ComFuncs
    from Common.Logger import Logger
except KeyboardInterrupt:
    sys.exit(1)

# Instantiate Globals
params = ""
logger = ""
lastError = ComDefs.SUCCESS
ipAddr = ComDefs.FALSE
standalone = ComDefs.FALSE
cleanExit = ComDefs.TRUE

def usage():
    '''
        Display the Usage Information to the screen
    '''
    print
    print "Usage: CiscoSMFlash.py [options]"
    print 
    print "Options:"
    print "  -i, --ipaddress=IPADDR   The hostname or dotted IP address of the I/O Module"
    print "  -s, --tftpserver=IPADDR  The dotted IP address of the TFTP server hosting"
    print "                           the new firmware image"
    print "  -n, --imagename=FILE     The path and name of the I/O Module firmware image"
    print "                           file on the TFTP server"
    print "  -u, --username=USERNAME  An administrator user name used to log into the I/O"
    print "                           Module"
    print "  -p, --password=PASSWORD  The password of the administrator user name used to"
    print "                           log into the I/O Module"
    print "  -h, --help, -?           Display this (help) message"
    print "  -v, --verbose            Increase the debug output level by one"
    print "  -V, --version            Display the CiscoSMFlash version number and exit"
    print "                           the same firmware version already installed"


def parse_args(opts):
    """
       Parse the arguments and options and handle them accordingly
    """
    global params, ipAddr, cleanExit

    for o, a in opts:
        if o in ("-?", "-h", "--help"):
            usage()
            return cleanExit
        elif o in ("-v", "--verbose"):
            params.debugLevel += 1
        elif o in ("-V", "--version"):
            print ComDefs.VERSION
            return cleanExit
        elif o in ("-i", "--ipaddress"):
            ipAddr = a
        elif o in ("-s", "--tftpserver"):
            params.tftpIPAddr = a
        elif o in ("-n", "--imagename"):
            params.io1Filename1 = a
        elif o in ("-u", "--username"):
            params.io1UserName = a
        elif o in ("-p", "--password"):
            params.io1Password = a
        else:
            usage()
            return cleanExit

    return ComDefs.SUCCESS

def verify_args():
    """
       Verify that the required arguments have been supplied.
    """

    global params, ipAddr, cleanExit

    if ipAddr == "":
        print "The hostname or dotted IP address of the I/O Module was not specified."
        usage()
        return cleanExit
    if params.tftpIPAddr == "":
        print "The dotted IP address of the TFTP server was not specified."
        usage()
        return cleanExit
    if params.io1Filename1 == "":
        print "The path and name of the I/O Module firmware image file on the TFTP"
        print "server was not specified."
        usage()
        return cleanExit

    return ComDefs.SUCCESS


def flash_switch(ioIPAddr, ioBay=1):
    """
       Flash the Cisco switch via interaction with the telnet menu interface.
    """

    global params, logger, standalone, lastError

    if standalone:
        logger.log("Updating Ethernet Switch Module firmware.", ComDefs.TRUE)

    # Get Variables from params class
    if ioBay == 1:
        userName = params.io1UserName
        password = params.io1Password
        filename = params.io1Filename1
    elif ioBay == 2:
        userName = params.io2UserName
        password = params.io2Password
        filename = params.io2Filename1
    elif ioBay == 3:
        userName = params.io3UserName
        password = params.io3Password
        filename = params.io3Filename1
    elif ioBay == 4:
        userName = params.io4UserName
        password = params.io4Password
        filename = params.io4Filename1

    if userName == "":
        logger.log("Error! Username was not supplied.")
        lastError = ComDefs.TN_INVALID_LOGIN
        return lastError
    if password == "":
        logger.log("Error! Password was not supplied.")
        lastError = ComDefs.TN_INVALID_LOGIN
        return lastError
    if filename == "":
        logger.log("Error! Firmware image filename was not supplied.")
        lastError = ComDefs.IO_INVALID_IMAGE_fILE
        return lastError

    # check to make sure tftpIPAddr is valid
    ipValid = re.compile('(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){2}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)')
    ipValidStr = ipValid.match(params.tftpIPAddr)
    if ipValidStr == None:
        logger.log("Error! Invalid TFTP Server IP address (%s)" % params.tftpIPAddr)
        lastError = ComDefs.TF_INVALID_IP
        return lastError
    
    # open a telnet session with the switch
    telnet = telnetlib.Telnet()
    lastError = connect(telnet, ioIPAddr, userName, password)
    if lastError != ComDefs.SUCCESS:
        return lastError
    
    # flash the firmware on the switch
    lastError = flash_firmware(telnet, filename)
    if lastError != ComDefs.SUCCESS:
        return lastError

    if standalone:
        logger.log("Success! Ethernet Switch Module update complete.", ComDefs.TRUE)

    lastError = ComDefs.SUCCESS
    return lastError
    
def connect(telnet, ioIPAddr, userName, password):
    """
        Connect and authenticate to the switch.
    """

    global params, logger

    try:
        telnet.open(ioIPAddr)
        logger.log("Connecting to %s" % ioIPAddr)
    except socket.error:
        logger.log("Socket error! Connection refused or timed out.")
        lastError = ComDefs.TN_SOCKET_ERR
        return lastError
    except KeyboardInterrupt:
        sys.exit(0)
    # look for "Username" and enter credentials
    reList = []
    reList.append(re.compile('Username'))
    reList.append(re.compile('>'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == 1:
        telnet.write("en\r")
        subreList = []
        subReList.append(re.compile('Username'))
        (subindex,submo,subtxt) = telnet.expect(subReList, ComDefs.TN_TIMEOUT)
        if index == -1:
            logger.log("Error! Unhandled timeout.")
            telnet.close()
            lastError = ComDefs.GENERAL_UPDATE_ERROR
            return lastError
    elif index == -1:
        logger.log("Error! Telnet timeout.")
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Connected to Ethernet Switch Module at (%s)." % ioIPAddr)
    telnet.write('%s\r' % userName)
    logger.log("Username sent.")
    # look for "Password" and enter credentials
    reList = []
    reList.append(re.compile('Password'))
    reList.append(re.compile('invalid'))
    (index,mo,txt) = telnet.expect(reList)
    if index == 1:
        logger.log("Error! Incorrect username.")
        telnet.close()
        lastError = ComDefs.TN_INVALID_LOGIN
        return lastError
    elif index == -1:
        logger.log("Error! Unhandled timeout.")
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    telnet.write('%s\r' % password)
    logger.log("Password sent.")
    # credentials are sent, continue to the next screen
    reList = []
    reList.append(re.compile('#'))
    reList.append(re.compile('invalid'))
    (index,mo,txt) = telnet.expect(reList)
    if index == 1:
        logger.log("Error! Password not accepted.")
        telnet.close()
        lastError = ComDefs.TN_INVALID_LOGIN
        return lastError
    logger.log("Username and Password accepted.")

    lastError = ComDefs.SUCCESS
    return lastError


def flash_firmware(telnet, filename):

    global params, logger

    # enter TFTP server IP and update image filename
    logger.log("Downloading Ethernet Switch Module firmware image to I/O Module.", ComDefs.TRUE)
    logger.log('archive download /overwrite tftp://%s/%s' % (params.tftpIPAddr, filename))
    telnet.write('archive download /overwrite tftp://%s/%s\r' % (params.tftpIPAddr, filename))
    # wait for update to complete or an error to occur
    reList = []
    reList.append(re.compile('!'))
    reList.append(re.compile('Unable'))
    reList.append(re.compile('Extracting\ files'))
    reList.append(re.compile('%(.*)\\n'))
    reList.append(re.compile('ERROR: (.*)\\n'))
    reList.append(re.compile('image\ installed'))
    while ComDefs.TRUE:
        (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
        if index == -1:
            logger.log("Error! Update Failed.")
            logger.log("Last message received:\n%s" % txt)
            telnet.close()
            lastError = ComDefs.GENERAL_UPDATE_ERROR
            return lastError
        elif index == 0:
            sys.stdout.write(".")
            sys.stdout.flush()
            continue
        elif index == 1:
            logger.log("Error! Unable to create directory.")
            telnet.write('exit\r')
            telnet.close()
            lastError = ComDefs.GENERAL_UPDATE_ERROR
            return lastError
        elif index == 2:
            print
            logger.log("Extracting files.", ComDefs.TRUE)
            continue
        elif index == 3:
            logger.log("%s" % mo.group(1))
            continue
        elif index == 4:
            logger.log("Error! %s" % mo.group(1))
            telnet.write('exit\r')
            telnet.close()
            lastError = ComDefs.IO_FIRMWARE_IMAGE_NOT_FOUND
            return lastError
        elif index == 5:
            print
            logger.log("Firmware image installed.", ComDefs.TRUE)
            break

    reList = []
    reList.append(re.compile('#'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Error! Update Failed.")
        logger.log("Last message received:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Writing to memory", ComDefs.TRUE)
    telnet.write('write memory\r')
    reList = []
    reList.append(re.compile('#'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Error! Update Failed.")
        logger.log("Last message received:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Reloading switch", ComDefs.TRUE)
    telnet.write('reload\r')
    reList = []
    reList.append(re.compile('confirm'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Error! Update Failed.")
        logger.log("Last message received:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    telnet.write('\r')
    telnet.close()

    lastError = ComDefs.SUCCESS
    return lastError
    

def main():
    """ Main function used to run this script in standalone mode"""

    # Global Variables
    global params, logger, ipAddr, standalone, lastError

    params = ComDefs.parameters()
    standalone = ComDefs.TRUE

    # Parse command line options with getopt()
    try:
        opts, args = getopt.getopt(sys.argv[1:], "?hvVi:s:n:u:p:",
             ["?", "help", "verbose", "version",
              "ipaddress=", "tftpserver=", "imagename=", "username=", "password="])

    except getopt.GetoptError:
        # Print usage information and exit
        usage()
        sys.exit(0)
    
    lastError = parse_args(opts)
    if lastError != ComDefs.SUCCESS:
        return lastError

    lastError = verify_args()
    if lastError != ComDefs.SUCCESS:
        return lastError
        
    logger = Logger(params)
    flash_switch(ipAddr)
    
# This get's invoked if they run this script directly
if __name__ == '__main__':
    main()
