#! /usr/bin/env python
###############################################################################
#
#  Script name                  : IOInfo.py
#
#  Dependencies                 : ComFuncs.py
#                                 ComDefs.py
#                                 MMComunications.py
#                    
#  Objective                    : Script used to login to the management port (6090)
#                                 of the MM and run dot commands to get information
#                                 about the I/O modules in the BladeCenter chassis
#                    
#  Written by                   : Mike Nolterieke
#                                 mikenol@us.ibm.com
#                     
#  Change History               : who -   when   - what
#                                 MHN - 07/01/04 - Initial revision.
#                                 RKC - 07/26/04 - use getopt instead of optparse
#                                 RKC - 07/26/04 - add logging
#  
#  Copyright IBM Corporation, 2004
#
###############################################################################
'''
   Script used to login to the management port (6090) of the MM and run dot commands
   to get information about the I/O modules in the BladeCenter chassis.
'''
try:
    import sys, getopt
    sys.path.append("..")
    from Common import ComDefs, ComFuncs
    from Common.MMCommunications import MMCom
    from Common.Logger import Logger
except KeyboardInterrupt:
    sys.exit(1)

# The last error that occurred
lastError = ComDefs.SUCCESS

# The MMCom class
mmCom = ""
logger = ""
standalone = ComDefs.FALSE
cleanExit = ComDefs.TRUE

class IOModuleInfo:
    isInstalled = ComDefs.FALSE
    ipAddress = 0
    manufacturer = ""
    codeLevel = ""
    moduleBay = 0
    disable = ComDefs.FALSE
    type = 0
    
    def __init__(self, bay):
        """
        	Constructor that probes provided bay, and if a I/O
        	module is installed, populates itself with pertinent
        	module information. 
        """
        self.isInstalled = getIsModuleInstalled(bay)
        if self.isInstalled:
            isInstalled = ComDefs.TRUE
            disable = ComDefs.FALSE
            self.moduleBay = bay
            self.ipAddress = getModuleIPAddress(bay)
            self.manufacturer = getManufacturerID(bay)
            self.codeLevel = getModuleCodeInfo(bay, 1, ComDefs.TRUE)
            self.type = getModuleType(bay)
            
            ipAddress = self.ipAddress
            manufacturer = self.manufacturer
            codeLevel = self.codeLevel
            moduleBay = self.moduleBay
            disable = self.disable
            type = self.type
            
    def toString(self):
        
    	""" Displays contents of class members """
        print ""
        print "*** I/O Module Information ***"
        print "Installed in bay: %s" % self.moduleBay
        print "IP Address: %s" % self.ipAddress
        print "Manufacturer: %s" % self.manufacturer
        print "Code Level: %s" % self.codeLevel
        print "Type: %s" % self.type
        print ""
        
        
def usage():
    print "Usage: IOInfo.py [options]"
    print "\nOptions:"
    print "  -i, --ipaddress=IPADDR   The hostname or dotted IP address of the"
    print "                           Management Module"
    print "  -u, --username=USERNAME  An administrator user name used to log into"
    print "                           the Management Module"
    print "  -p, --password=PASSWORD  The password of the administrator username"
    print "                           used to log into the Management Module"
    print "  -h, --help, -?           Display this (help) message"
    print "  -v, --verbose            Increase the debug output level by one"
    print "  -V, --version            Display the IOInfo version number and exit"


def parse_args(opts, params):
    '''
       Parse the arguments and options and handle them accordingly
    '''

    global cleanExit

    for o, a in opts:
        if o in ("-?", "-h", "--help"):
            usage()
            return cleanExit
        elif o in ("-v", "--verbose"):
            params.debugLevel += 1
        elif o in ("-V", "--version"):
            print ComDefs.VERSION
            return cleanExit
        elif o in ("-i", "--ipaddress"):
            params.mmIPAddress = a
        elif o in ("-u", "--username"):
            params.mmUserName = a
        elif o in ("-p", "--password"):
            params.mmPassword = a
        else:
            usage()
            return cleanExit

    return ComDefs.SUCCESS


def verify_args(params):
    '''
       Verify that the required arguments have been supplied.
    '''

    global cleanExit

    if params.mmIPAddress == "":
        print "The hostname or dotted IP address of the Management Module was not specified."
        usage()
        return cleanExit

    return ComDefs.SUCCESS


def connectToMM(params):
    '''
        Connect to the MM management port (6090) and login.
    '''
    # Use global variable
    global mmCom
    return (mmCom.open(ComDefs.TRUE, params.mmIPAddress, 6090, 5, params.mmUserName, params.mmPassword))

def getIsModuleInstalled(ioBay=1):
    '''
        Determine is the an I/O Module is installed in the specified bay (1-4, 1 by
        default) of the BladeCenter chassis.
    
        Returns:
        True if the I/O module is installed.
        If False is returned, you should check the global lastError variable to make
        sure it is set to ComDefs.SUCCESS.  This will ensure that there was not an
        error querying the MM and there really is not an I/O module installed in the
        specified bay.
    '''
    global mmCom, lastError

    # Is the module installed, default to False
    isInstalled = ComDefs.FALSE
    # For my own sanity...
    # The response we will get back from the MM will be 16 bytes
    # To determine if an I/O module is installed, we must check the bits of the 12'th byte
    # If an I/O module is installed, the bit will be 1
    # If an I/O module is not installed, the bit will be 0
    # The bit format is:
    # Bit -      8   7   6   5  4   3   2   1
    # Value -   128 64  32  16  8   4   2   1
    # There are four I/O module bays so we have to check bits 1, 2, 4, and 8
    # So, if the 12'th byte of the response is 0x07, the bits will be 0000 0111
    # Meaning that there are I/O modules installed in bays 1, 2, and 3 but not 4
    
    # Make sure we have a valid bay number
    if ((ioBay < 1) | (ioBay > 4)):
        lastError = ComDefs.MM_INVALID_SLOT_NUM
        return isInstalled
    # Convert the specified ioBay to the proper hex number so we can check the bit
    # in the response, default to 1
    slotNum = ComFuncs.intToHex(1, ComDefs.FALSE)
    if ioBay == 1:
        pass
    elif ioBay == 2:
        slotNum = ComFuncs.intToHex(2, ComDefs.FALSE)
    elif ioBay == 3:
        slotNum = ComFuncs.intToHex(4, ComDefs.FALSE)
    elif ioBay == 4:
        slotNum = ComFuncs.intToHex(8, ComDefs.FALSE)

    # Send the command and check for an error
    lastError = mmCom.sendCmd('R', '\x02\x05\x02\x01\x01', '')
    
    if lastError == ComDefs.SUCCESS:
        # Get the response and check the bits
        response = mmCom.getResponse()
        # AND (&) the 12'th byte of the response with the correct bit
        # If the bit is nonzero, there is an I/O module installed
        if (ord(response[12]) & slotNum) != 0:
            isInstalled = ComDefs.TRUE
    return isInstalled

def getManufacturerID(ioBay=1):
    '''
        Get the Manufacturer ID of the I/O Module in the specified bay (1-4, 1 by default)
        of the BladeCenter chassis.
    
        Returns:
        The 4 character Manufacturer ID (in text - DLNK,   NT, etc...) of the
        I/O module in the specified bay, or an empty string if there was an error.
        If an empty string is returned, the error code will be stored in the lastError
        global variable.
    '''
    global mmCom, lastError
    manID = ""
    
    # Make sure we have a valid bay number
    if ((ioBay < 1) | (ioBay > 4)):
        lastError = ComDefs.MM_INVALID_SLOT_NUM
        return manID
    # The I/O module bay to get the manufacturer ID
    slotNum = ComFuncs.intToHex(ioBay, ComDefs.TRUE)
    
    # Just for my own sanity...
    # The VPD Cache dot command 4.3.5.5 isn't supported in all levels of the MM firmware
    # 22.3.3.1 = VPD dot command
    # Data for the command is 6 bytes:
    #   Module Type         - 1 byte
    #   Slot Number         - 1 byte
    #   Read Address LSB    - 1 byte
    #   Read Address MSB    - 1 byte
    #   Read Length LSB     - 1 byte
    #   Read Length MSB     - 1 byte
    # Module Type   = /x02  - I/O module
    # Slot Number   =       - Determined by ioBay parameter
    # Read Address  = /x0060- \x00 is MSB and \x60 is LSB
    #   These bytes must be swapped when sending the command
    #   So, \x0060 becomes \x60\x00 in LSB, MSB format
    # Read Length   = \x0004- \x00 is MSB and \x04 is LSB
    #   These bytes must also be swapped when sending the command
    #   So, \x0004 becomes \x04\x00 in LSB, MSB format
    
    # Send the command and check for an error
    lastError = mmCom.sendCmd('R', '\x16\x03\x03\x01', slotNum + '\x60\x00\x04\x00')
    if lastError == ComDefs.SUCCESS:
        # Get the response and convert it to something useful (not hex)
        response = mmCom.getResponse()
        i = 0
        while i < 4:
            # Skip blank spaces so it can be compared later
            if response[i] == ' ':
                pass
            elif response[i] == '\0':
                break
            else:
                manID += "%s" % response[i]
            i += 1
    else: 
        print lastError
    # Return the manufacturer ID
    return manID


def getModuleType(ioBay=1):
    '''
        Get the Module Type of the I/O Module in the specified bay (1-4, 1 by default)
        of the BladeCenter chassis.
    
        Returns:
        The 1 character hex value associated with a module type of the
        I/O module in the specified bay, or an empty string if there was an error.
        If an empty string is returned, the error code will be stored in the lastError
        global variable.
    '''
    global mmCom, lastError
    modtype = ""
    
    # Make sure we have a valid bay number
    if ((ioBay < 1) | (ioBay > 4)):
        lastError = ComDefs.MM_INVALID_SLOT_NUM
        return modtype 
    # The I/O module bay to get the type
    slotNum = ComFuncs.intToHex(ioBay, ComDefs.TRUE)

    # Just for my own sanity...
    # 22.3.3.1 = VPD dot command
    # Data for the command is 6 bytes:
    #   Module Type         - 1 byte
    #   Slot Number         - 1 byte
    #   Read Address LSB    - 1 byte
    #   Read Address MSB    - 1 byte
    #   Read Length LSB     - 1 byte
    #   Read Length MSB     - 1 byte
    # Module Type   = /x02  - I/O module
    # Slot Number   =       - Determined by ioBay parameter
    # Read Address  = /x00AF- \x00 is MSB and \xAF is LSB
    #   These bytes must be swapped when sending the command
    #   So, \x00AF becomes \xAF\x00 in LSB, MSB format
    # Read Length   = \x0001- \x00 is MSB and \x01 is LSB
    #   These bytes must also be swapped when sending the command
    #   So, \x0001 becomes \x01\x00 in LSB, MSB format
    
    # send command
    lastError = mmCom.sendCmd('R', '\x16\x03\x03\x01', slotNum + '\xAF\x00\x01\x00')
    if lastError == ComDefs.SUCCESS:
        # Get the response and convert it to something useful (not hex)
        response = mmCom.getResponse()
        # we actually want the hex value after all
        modtypeval = ord(response[0])
        if modtypeval == 0x11:
            modtype = "Ethernet Switch Module"
        elif modtypeval == 0x12:
            modtype = "Fibre Channel Switch Module"
        elif modtypeval == 0x14:
            modtype = "Infiniband Switch Module"
        elif modtypeval == 0x15:
            modtype = "Scalability Port Switch Module"
        elif modtypeval == 0x16:
            modtype = "Mixed Protocol Switch Module"
        elif modtypeval == 0x1E:
            modtype = "Optical Pass-thru Module"
        elif modtypeval == 0x1F:
            modtype = "Serial Port Concentrator Module"
        elif modtypeval == 0x28:
            modtype = "Copper Pass-thru Module"
        elif modtypeval == 0x29:
            modtype = "Cirpack Clock Dist Module"
        else:
            modtype = "Unknown I/O Module"
    else:
        print lastError
    # Return the type
    return modtype


def getModuleIPAddress(ioBay=1):
    ''' Get the IP address of the I/O Module in the specified bay (1-4, 1 by default)
        of the BladeCenter chassis.

        Returns:
        The dotted IP address (in text - 1.1.1.1) of the I/O module in the specified
        bay, or an empty string if there was an error.  If an empty string is returned, 
        the error code will be stored in the lastError global variable.
    '''
    global mmCom, lastError
    ipAddr = ""
    
    # Make sure we have a valid bay number
    if ((ioBay < 1) | (ioBay > 4)):
        lastError = ComDefs.MM_INVALID_SLOT_NUM
        return ipAddr
    # The I/O module bay to get the IP address
    slotNum = ComFuncs.intToHex(ioBay, ComDefs.TRUE)
    
    # Just for my own sanity...
    # 4.3.5.19.{slotNum} = I/O Module VPD dot command
    # Data for the command is 4 bytes:
    #   Read Address LSB    - 1 byte
    #   Read Address MSB    - 1 byte
    #   Read Length LSB     - 1 byte
    #   Read Length MSB     - 1 byte
    # Read Address  = typically 0400 + 010F
    #   These bytes must be added and then swapped when sending the command
    #   So, 0400 + 010F becomes 050F which is \x0F\x05 in LSB, MSB format
    # Read Length   = 48 decimal = 0030 hex
    #   These bytes must also be swapped when sending the command
    #   So, \x0030 becomes \x30\x00 in LSB, MSB format
    
    # Get the VPD base offset - hardcoded for now
    response = ['','\x04','\x00']
    # Check for an error
    if response == "":
        return ipAddr

    # Put the data string together that we pass to the MM
    # We must add 0x0F to the LSB and 0x01 to the MSB of the VPD base offset to get the
    # IP address of the specified module
    # The %c converts the resulting numbers into an actual character (not text) that
    # will be sent to the MM
    data = '%c%c\x30\x00' % ((ord(response[2]) + 0X0F), (ord(response[1]) + 0X01))

    # Send the command and check for an error
    lastError = mmCom.sendCmd('R', '\x04\x03\x05\x13' + slotNum, data)
    if lastError == ComDefs.SUCCESS:
        # Get the response and convert it to something useful (not hex)
        response = mmCom.getResponse()
        # Get just the first IP address (the first 4 bytes of the response)
        i = 0
        while i < 4:
            ipAddr += "%d" % ord(response[i])
            if i < 3:
                ipAddr += "."
            i += 1
    # Return the IP address
    return ipAddr

def getModuleCodeInfo(ioBay=1, codeLevel=1, isBuildID=ComDefs.TRUE):
    '''
        Get the Build ID or Revision Level of the I/O Module in the specified bay
        (1-4, 1 by default) of the BladeCenter chassis.
        There can be up to 5 different code levels running on any given I/O Module.
        Use the codeLevel variable to specify which Build ID (1 - 5) you wish to obtain
        Specify False for the isBuildID argument to get the revision level.
    
        Returns:
        The specified Build ID or Revision Level (as text without the trailing blanks)
        of the specified I/O Module or an empty string if there was no data or an error.
        If an empty string is returned, the error code will be stored in the lastError
        global variable if there was actually an error.
        If the lastError variable is ComDefs.SUCCESS, it means that there was not an error,
        the Build ID or Revision Level was filled with all blank spaces.
    '''
    global mmCom, lastError

    codeInfo = ""
    # Make sure we have a valid bay number
    if ((ioBay < 1) | (ioBay > 4)):
        lastError = ComDefs.MM_INVALID_SLOT_NUM
        return codeInfo
    # The I/O module bay to get the info for
    slotNum = ComFuncs.intToHex(ioBay, ComDefs.TRUE)
    
    # The offsets (LSB, MSB) of which code level to get, default to code level 1
    codeLevLSB = 0X06
    codeLevMSB = 0X00
    if codeLevel == 1:
        pass
    elif codeLevel == 2:
        codeLevLSB = 0X2A
        codeLevMSB = 0X00
    elif codeLevel == 3:
        codeLevLSB = 0X4E
        codeLevMSB = 0X00
    elif codeLevel == 4:
        codeLevLSB = 0X72
        codeLevMSB = 0X00
    elif codeLevel == 5:
        codeLevLSB = 0X96
        codeLevMSB = 0X00
    else:
        lastError = ComDefs.MM_INVALID_CODE_LEVEL
        return codeInfo
    
    # Just for my own sanity...
    # 4.3.5.19.{slotNum} = I/O Module VPD dot command
    # Data for the command is 4 bytes:
    #   Read Address LSB    - 1 byte
    #   Read Address MSB    - 1 byte
    #   Read Length LSB     - 1 byte
    #   Read Length MSB     - 1 byte
    # Read Address  = typically 0400 + 0006
    #   These bytes must be added and then swapped when sending the command
    #   So, 0400 + 0006 becomes 0406 which is \x06\x04 in LSB, MSB format
    # Read Length   = 36 decimal = 0024 hex
    #   These bytes must also be swapped when sending the command
    #   So, \x0024 becomes \x24\x00 in LSB, MSB format
    
    # Get the VPD base offset
    # The VPD Cache dot command 4.3.5.5 isn't supported in all levels of the MM firmware
    # hardcoded for now
    response = ['','\x04','\x00']
    # Check for an error
    if response == "":
        return codeInfo

    # Put the data string together that we pass to the MM
    # We must add 0x06 to the LSB and 0x00 to the MSB of the VPD base offset to get the
    # information of the specified module
    # The %c converts the resulting numbers into an actual character (not text) that
    # will be sent to the MM
    data = '%c%c\x30\x00' % ((ord(response[2]) + codeLevLSB), (ord(response[1]) + codeLevMSB))

    # Send the command and check for an error
    lastError = mmCom.sendCmd('R', '\x04\x03\x05\x13' + slotNum, data)
    if lastError == ComDefs.SUCCESS:
        # Get the response and convert it to something useful (not hex)
        response = mmCom.getResponse()
        # Determine what we need to get (Build ID or Revision Level)
        i = 0
        # Get the Build ID
        if isBuildID:
            # Get just the 10 byte build ID (bytes 4 - 13 of the response)
            i = 4
            while i < 14:
                # Skip any blank spaces
                if response[i] == " ":
                    pass
                else:
                    codeInfo += "%s" % response[i]
                i += 1
        # Get the Revision Level
        else:
            # Get just the 4 byte revision level (bytes 0 - 3 of the response)
            while i < 4:
                # Skip any blank spaces
                if response[i] == " ":
                    pass
                else:
                    codeInfo += "%s" % response[i]
                i += 1
    # Return the information
    return codeInfo


def main():
    '''
        Main function used to run this script in standalone mode
    '''
    # Define using a global variable
    global mmCom, lastError, logger

    params = ComDefs.parameters()
    standalone = ComDefs.TRUE

    # Parse command line options with getopt()
    try:
        opts, args = getopt.getopt(sys.argv[1:], "?hvVi:u:p:",
             ["?", "help", "verbose", "version", "ipaddress=", "username=", "password="])

    except getopt.GetoptError:
        # Print usage information and exit
        usage()
        sys.exit(0)

    lastError = parse_args(opts, params)
    if lastError != ComDefs.SUCCESS:
        return lastError

    lastError = verify_args(params)
    if lastError != ComDefs.SUCCESS:
        return lastError

    logger = Logger(params)
    # See if we are in debug mode
    if params.debugLevel == 0:
        # Instanciate the class and leave debug off
        mmCom = MMCom()
    else:
        # Instanciate the class and turn debug on
        mmCom = MMCom(ComDefs.TRUE)
    mmCom.logger = logger

    # Get the I/O module info using the 6090 management port
    # Connect to the MM port 6090
    logger.log("Retrieving I/O Module info from Management Module", ComDefs.TRUE)
    logger.log("Opening connection and logging into the MM")
    if connectToMM(params) != ComDefs.SUCCESS:
        logger.log("There was an error connecting or logging into the MM!", ComDefs.TRUE)
        sys.exit(1)
    else:
        logger.log("Successfully connected and logged into the MM!")

    # The VPD Cache dot command 4.3.5.5 isn't supported in all levels of the MM firmware
    #getVPDBaseOffset()
    getModuleIPAddress()

    logger.log("Getting BladeCenter I/O Module information...")
    # Loop through the I/O modules and display the information
    i = 1
    while i < 5:
        logger.log("I/O Module %d Information:" % i, ComDefs.TRUE)
        # See if the module is installed
        isInstalled = getIsModuleInstalled(i)
        # If it is installed
        if isInstalled:
            logger.log("\tIs Installed:\t\tTRUE", ComDefs.TRUE)
            manID = getManufacturerID(i)
            if manID == "":
                manID = "Unknown"
            logger.log("\tManufacturer ID:\t%s" % manID, ComDefs.TRUE)
            ipAddr = getModuleIPAddress(i)
            if ipAddr == "":
                ipAddr = "Unknown"
            logger.log("\tIP Address:\t\t%s" % ipAddr, ComDefs.TRUE)
            ioType = getModuleType(i)
            logger.log("\tModule Type:\t\t%s" % ioType, ComDefs.TRUE)
            # Loop through and get all of the possible code levels
            j = 1
            while j < 6:
                # Get the build ID
                buildID = getModuleCodeInfo(i, j)
                # If the build ID is blank, make sure it was not an error
                if buildID == "":
                    # If it was an error, we will display Unknown...
                    if lastError != ComDefs.SUCCESS:
                        buildID = "Unknown"
                # If the build ID is still blank, don't try to display it
                if buildID != "":
                    logger.log("\tFirmware %d Info:" % j, ComDefs.TRUE)
                    logger.log("\t\tBuild ID:\t%s" % buildID, ComDefs.TRUE)
                    # Get the code level revision number
                    revision = getModuleCodeInfo(i, j, ComDefs.FALSE)
                    # If the revision is blank, make sure it was not an error
                    if revision == "":
                        # If it was an error, we will display Unknown...
                        if lastError != ComDefs.SUCCESS:
                            revision = "Unknown"
                    logger.log("\t\tRevision Level:\t%s" % revision, ComDefs.TRUE)
                j += 1
        # If it is not installed
        else:
            # It returned False, we must make sure there was not an error first
            if lastError == ComDefs.SUCCESS:
                logger.log("\tIs Installed:\t\tFALSE", ComDefs.TRUE)
            else:
                logger.log("\tCould not determine if an I/O module is installed in bay %d!" % i, ComDefs.TRUE)
        i += 1

    # Close the connection to the MM
    mmCom.close()
    
# This get's invoked if they run this script directly
if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        ComFuncs.keyboardISR(logger)

