#! /usr/bin/env python
#NT
###############################################################################
#
#  Script name                  : NortelSMFlash.py
#
#  Dependencies                 : ComDefs.py
#                                 ComFuncs.py
#                                 Logger.py
#                    
#  Objective                    : Script used to login to a Nortel switch 
#                                 using telnet and update the firmware via a 
#                                 TFTP server.
#                    
#  Written by                   : Robert K. Christner
#                                 christne@us.ibm.com
#                     
#  Change History               : who -   when   - what
#                                 RKC - 07/19/04 - Initial revision.
#                                 RKC - 08/01/04 - use getopt instead of optparse
#  
#  Copyright IBM Corporation, 2004
#
###############################################################################
"""
   Script used to login to a Nortel switch over telnet and update the
   firmware via a TFTP server.
"""
try:
    import time, sys, os, re, telnetlib, getopt, socket
    sys.path.append("..")
    from Common import ComDefs, ComFuncs
    from Common.Logger import Logger
except KeyboardInterrupt:
    sys.exit(1)

# Instantiate globals
params = ""
logger = ""
ipAddr = ""
standalone = ComDefs.FALSE
lastError = ComDefs.SUCCESS
cleanExit = ComDefs.TRUE

def usage():
    '''
        Display the Usage Information to the screen
    '''

    print "Usage: NortelSMFlash.py [options]"
    print "\nOptions:"
    print "  -i, --ipaddress=IPADDR   The hostname or dotted IP address of the I/O Module"
    print "  -s, --tftpserver=IPADDR  The dotted IP address of the TFTP server hosting the"
    print "                           new firmware image"
    print "  -b, --bootimage=FILE     The path and name of the I/O Module firmware boot"
    print "                           image file on the TFTP server"
    print "  -o, --osimage=FILE       The path and name of the I/O Module firmware OS"
    print "                           image file on the TFTP server"
    print "  -u, --username=USERNAME  An administrator user name used to log into the I/O"
    print "                           Module"
    print "  -p, --password=PASSWORD  The password of the administrator user name used to"
    print "                           log into the I/O Module"
    print "  -h, --help, -?           Display this (help) message"
    print "  -v, --verbose            Increase the debug output level by one"
    print "  -V, --version            Display the NortelSMFlash version number and exit"


def parse_args(opts):
    """
       Parse the arguments and options and handle them accordingly
    """

    global params, ipAddr, cleanExit

    for o, a in opts:
        if o in ("-?", "-h", "--help"):
            usage()
            return cleanExit
        elif o in ("-v", "--verbose"):
            params.debugLevel += 1
        elif o in ("-V", "--version"):
            print ComDefs.VERSION
            return cleanExit
        elif o in ("-i", "--ipaddress"):
            ipAddr = a
        elif o in ("-s", "--tftpserver"):
            params.tftpIPAddr = a
        elif o in ("-o", "--osimage"):
            params.io1Filename1 = a
        elif o in ("-b", "--bootimage"):
            params.io1Filename2 = a
        elif o in ("-u", "--username"):
            params.io1UserName = a
        elif o in ("-p", "--password"):
            params.io1Password = a
        else:
            usage()
            return cleanExit

    return ComDefs.SUCCESS


def verify_args():
    """
       Verify that the required arguments have been supplied.
    """

    global params, ipAddr, cleanExit

    if ipAddr == "":
        print "The hostname or dotted IP address of the I/O Module was not specified."
        usage()
        return cleanExit
    if params.tftpIPAddr == "":
        print "The hostname or dotted IP address of the TFTP server was not specified."
        usage()
        return cleanExit
    if params.io1Filename1 == "":
        print "The path and name of the I/O Module boot firmware image file on the TFTP"
        print "server was not specified."
        usage()
        return cleanExit
    if params.io1Filename2 == "":
        print "The path and name of the I/O Module OS firmware image file on the TFTP"
        print "server was not specified."
        usage()
        return cleanExit

    return ComDefs.SUCCESS

def reset_switch(telnet):

    # Global Variables
    global params, logger, standalone

    # reset
    telnet.write('reset\r')
    reList = []
    reList.append(re.compile('Confirm\ reset'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    telnet.write('y\r')
    telnet.close()
    logger.log("The switch was reset.")

    # make sure everything gets printed to the screen
    sys.stdout.flush()

    lastError = ComDefs.SUCCESS
    return lastError

def flash_boot(telnet, image):

    # Global Variables
    global params, logger, standalone

    # update the boot image
    logger.log("Updating the Ethernet Switch Module boot firmware.", ComDefs.TRUE)
    logger.log("Updating firmware from %s, with %s." % (params.tftpIPAddr, image))
    logger.log('/boot/gtimg boot %s %s\r' % (params.tftpIPAddr, image))
    telnet.write('/boot/gtimg boot %s %s\r' % (params.tftpIPAddr, image))
    # Confirm download operation
    reList = []
    reList.append(re.compile('Confirm\ download'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    telnet.write('y\r')
    #logger.log("Confirmed download.")
    # Validate file
    reList = []
    reList.append(re.compile('ERROR: (.*)\\n'))
    reList.append(re.compile('appears\ valid'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.write('exit\r')
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    elif index == 0:
        logger.log("Error! %s" % mo.group(1))
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    elif index == 1:
        logger.log("File seems to be valid.")
    # Download file
    reList = []
    reList.append(re.compile('progress'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Boot image download in progress.", ComDefs.TRUE)
    # Show progress
    counter = 0
    while 1:
        reList = []
        reList.append(re.compile('\.'))
        reList.append(re.compile('[^.]'))
        (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
        if index == 0:
            if counter % 10 == 0:
                sys.stdout.write(".")
                sys.stdout.flush()
            counter += 1
            continue
        elif index == -1:
            logger.log("Error! TFTP download timed out.")
            telnet.close()
            lastError = ComDefs.TF_DOWNLOAD_TIMEOUT
            return lastError
        else:
            print
            logger.log("Boot image (FS) download complete.")
            logger.log("Writing FS Sector in FLASH.", ComDefs.TRUE)
            break
    # FS
    # FLASH Complete
    reList = []
    reList.append(re.compile('contains'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT_LONG)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("FS Sector write complete.")

    # FPGA
    reList = []
    reList.append(re.compile('wait'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Boot image (FPGA) download complete.")
    logger.log("Writing FPGA sector in FLASH.", ComDefs.TRUE)
    # FLASH Complete
    reList = []
    reList.append(re.compile('contains'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT_LONG)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("FPGA Sector write complete.")

    # Kernel
    reList = []
    reList.append(re.compile('wait'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Boot image (Kernel) download complete.")
    logger.log("Writing Kernel sector in FLASH.", ComDefs.TRUE)
    # FLASH Complete
    reList = []
    reList.append(re.compile('contains'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT_LONG)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Kernel Sector write complete.")

    # Boot
    reList = []
    reList.append(re.compile('wait'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Boot image (Boot) download complete.")
    logger.log("Writing Boot sector in FLASH.", ComDefs.TRUE)
    # FLASH Complete
    reList = []
    reList.append(re.compile('contains'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT_LONG)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Boot Sector write complete.")

    reList = []
    reList.append(re.compile('#'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Ethernet Switch Module boot firmware successfully updated.", ComDefs.TRUE)

    lastError = ComDefs.SUCCESS
    return lastError

def flash_os(telnet, bank, image):

    # Global Variables
    global params, logger, standalone

    # update the OS image
    logger.log("Updating the Ethernet Switch Module OS firmware.", ComDefs.TRUE)
    logger.log("Updating bank %s, from %s, with %s." % (bank, params.tftpIPAddr, image))
    logger.log('/boot/gtimg image%s %s %s\r' % (bank, params.tftpIPAddr, image))
    telnet.write('/boot/gtimg image%s %s %s\r' % (bank, params.tftpIPAddr, image))
    # Confirm download operation
    reList = []
    reList.append(re.compile('Confirm\ download'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    telnet.write('y\r')
    #logger.log("Confirmed download.")
    # Validate file
    reList = []
    reList.append(re.compile('ERROR: (.*)\\n'))
    reList.append(re.compile('appears\ valid'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.write('exit\r')
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    elif index == 0:
        logger.log("Error! %s" % mo.group(1))
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    elif index == 1:
        logger.log("File seems to be valid.")
    # Download file
    reList = []
    reList.append(re.compile('progress'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("OS image download in progress.", ComDefs.TRUE)
    # Show progress
    counter = 0
    while 1:
        reList = []
        reList.append(re.compile('\.'))
        reList.append(re.compile('[^.]'))
        (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
        if index == 0:
            if counter % 10 == 0:
                sys.stdout.write(".")
                sys.stdout.flush()
            counter += 1
            continue
        elif index == -1:
            logger.log("Error! TFTP download timed out.")
            telnet.close()
            lastError = ComDefs.TF_DOWNLOAD_TIMEOUT
            return lastError
        else:
            print
            logger.log("OS image download complete.")
            logger.log("Writing OS image to FLASH.", ComDefs.TRUE)
            break
    # FLASH Complete, verifying
    reList = []
    reList.append(re.compile('complete'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT_LONG)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("Write complete. Now verifying FLASH.")
    # verified
    reList = []
    reList.append(re.compile('successful'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    logger.log("FLASH successfully verified.")

    # Do you want to change that to the new image2
    reList = []
    reList.append(re.compile('#'))
    reList.append(re.compile('change\ that\ to\ the'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    elif index == 1:
        telnet.write('y\r')
        logger.log("Switched to the new image bank.")

    logger.log("Ethernet Switch Module OS firmware successfully updated.", ComDefs.TRUE)

    lastError = ComDefs.SUCCESS
    return lastError

def connect(telnet, ioIPAddr, userName, password):
    """
        Connect and authenticate to the switch.
    """

    # Global Variables
    global params, logger, standalone

    logger.log("Connecting to Ethernet Switch Module (%s)." % ioIPAddr)
    try:
        telnet.open(ioIPAddr)
    except socket.error:
        logger.log("Socket error! Connection refused or timed out.")
        lastError = ComDefs.TN_SOCKET_ERR
        return lastError
    except KeyboardInterrupt:
        sys.exit(0)

    # look for "username" or "password" and enter credentials
    reList = []
    reList.append(re.compile('username'))
    reList.append(re.compile('password'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == 0:
        # if we were prompted for username, send it
        telnet.write('%s\r' % userName)
        logger.log("Username sent.")
        # now look for a password prompt
        reSubList = []
        reSubList.append(re.compile('password'))
        reSubList.append(re.compile('incorrect'))
        (subIndex,subMo,subTxt) = telnet.expect(reSubList, 20)
        if subIndex == 1:
            logger.log("Error! Incorrect username.")
            telnet.close()
            lastError = ComDefs.TN_INVALID_LOGIN
            return lastError
        logger.log("Username accepted.")
    elif index == -1:
        logger.log("Telnet connection timeout")
        lastError = ComDefs.IO_FAILED_TO_CONNECT
        return lastError
    # at this point we need to enter a password
    telnet.write('%s\r' % password)
    logger.log("Password sent.")
    reList = []
    reList.append(re.compile('#'))
    reList.append(re.compile('incorrect'))
    reList.append(re.compile('Set\ Up'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == -1:
        logger.log("Unhandled timout.")
        logger.log("Last message recieved:\n%s" % txt)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    elif index == 1:
        logger.log("Error! Incorrect password.")
        telnet.close()
        lastError = ComDefs.TN_INVALID_LOGIN
        return lastError
    elif index == 2:
        logger.log("The switch has not been setup, bypassing...")
        telnet.write('n\r')

    lastError = ComDefs.SUCCESS
    return lastError


def flash_switch(ioIPAddr, ioBay=1, bank=2):
    """
       Function used to actually flash the Nortel switch via interaction with
       the telnet menu interface.
    """

    # Global Variables
    global params, logger, standalone

    if standalone:
        logger.log("Updating Ethernet Switch Module firmware.", ComDefs.TRUE)

    # Get Variables from params class
    if ioBay == 1:
        userName = params.io1UserName
        password = params.io1Password
        osImage = params.io1Filename1
        bootImage = params.io1Filename2
    elif ioBay == 2:
        userName = params.io2UserName
        password = params.io2Password
        osImage = params.io2Filename1
        bootImage = params.io2Filename2
    elif ioBay == 3:
        userName = params.io3UserName
        password = params.io3Password
        osImage = params.io3Filename1
        bootImage = params.io3Filename2
    elif ioBay == 4:
        userName = params.io4UserName
        password = params.io4Password
        osImage = params.io4Filename1
        bootImage = params.io4Filename2

    # a blank username is valid
    if password == "":
        logger.log("Error! Password was not supplied.")
        lastError = ComDefs.TN_INVALID_LOGIN
        return lastError
    if osImage == "":
        logger.log("Error! Firmware OS image filename was not supplied.")
        lastError = ComDefs.IO_INVALID_IMAGE_fILE
        return lastError
    if bootImage == "":
        logger.log("Error! Firmware BOOT image filename was not supplied.")
        lastError = ComDefs.IO_INVALID_IMAGE_fILE
        return lastError


    # check to make sure tftpIPAddr is valid
    ipValid = re.compile('(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){2}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)')
    ipValidStr = ipValid.match(params.tftpIPAddr)
    if ipValidStr == None:
        logger.log("Error! Invalid TFTP Server IP address (%s)" % params.tftpIPAddr)
        lastError = ComDefs.TF_INVALID_IP
        return lastError

    # see if a hostname or ip address was entered for the switch
    #ipValidStr = ipValid.match(ioIPAddr)
    #if ipValidStr:
        # try to get the hostname to make sure address exists on the network
    #    try:
    #        ioHostname = socket.gethostbyaddr(ioIPAddr)
    #    except:
    #        logger.log("Error! Invalid IP address for telnet connection.")
    #        lastError = ComDefs.TN_BAD_IP
    #        return lastError

    # open a telnet session with the switch
    telnet = telnetlib.Telnet()
    lastError = connect(telnet, ioIPAddr, userName, password)
    if lastError != ComDefs.SUCCESS:
        return lastError
    
    # flash the OS image on the switch
    lastError = flash_os(telnet, bank, osImage)
    if lastError != ComDefs.SUCCESS:
        return lastError

    # flash the boot image on the switch
    lastError = flash_boot(telnet, bootImage)
    if lastError != ComDefs.SUCCESS:
        return lastError

    if standalone:
        logger.log("Success! Ethernet Switch Module firmware update complete.", ComDefs.TRUE)
    lastError = ComDefs.SUCCESS
    return lastError


def main():
    """
       Main function used to run this script in standalone mode
    """

    # Global Variables
    global params, logger, standalone, ipAddr

    params = ComDefs.parameters()
    standalone = ComDefs.TRUE
    
    # Parse command line options with getopt()
    try:
        opts, args = getopt.getopt(sys.argv[1:], "?hvVi:s:b:o:u:p:",
             ["?", "help", "verbose", "version", "ipaddress=", "tftpserver=", "bootimage=", "osimage=", "username=", "password="])

    except getopt.GetoptError:
        # Print usage information and exit
        usage()
        sys.exit(0)

    lastError =  parse_args(opts)
    if lastError != ComDefs.SUCCESS:
        return lastError

    # Validate the input
    lastError = verify_args()
    if lastError != ComDefs.SUCCESS:
        return lastError
        
    logger = Logger(params)
    flash_switch(ipAddr)
    
# This get's invoked if they run this script directly
if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        ComFuncs.keyboardISR(logger)
