#! /usr/bin/env python
#IBM
###############################################################################
#
#  Script name                  : PMFlash.py
#
#  Dependencies                 : ComDefs.py
#                                 ComFuncs.py
#                                 Logger.py
#                    
#  Objective                    : Script used to login to a pass-thru module 
#                                 and update the firmware via the Management
#                                 Module.
#                    
#  Written by                   : John R. Hicks
#                                 jrhicks@us.ibm.com
#                     
#  Change History               : who -   when   - what
#                                 JRH - 10/1/2004 - Initial revision.
#  
#  Copyright IBM Corporation, 2004
#
###############################################################################
"""
    Script used to login to a pass-thru switch module 
    and update the firmware via the Management
    Module.
"""

try:
    import time, sys, os, os.path, re, getopt, telnetlib, socket
    sys.path.append("..")
    from Common import ComDefs, ComFuncs
    from Common.Logger import Logger
except KeyboardInterrupt:
    sys.exit(1)

# Instantiate Globals
params = ""
logger = ""
lastError = ComDefs.SUCCESS
cleanExit = ComDefs.TRUE
ipAddr = ""
standalone = ComDefs.FALSE

def usage():
    '''
        Display the Usage Information to the screen
    '''
    print
    print "Usage: PMFlash.py [options]"
    print 
    print "Options:"
    print "  -i, --ipaddress=IPADDR   The hostname or dotted IP address of the I/O Module"
    print "  -s, --tftpserver=IPADDR  The dotted IP address of the TFTP server hosting"
    print "                           the new firmware image"
    print "  -n, --imagename=FILE     The path and name of the I/O Module firmware image"
    print "                           file on the TFTP server"
    print "  -u, --username=USERNAME  An administrator user name used to log into the I/O"
    print "                           Module"
    print "  -p, --password=PASSWORD  The password of the administrator user name used to"
    print "                           log into the I/O Module"
    print "  -h, --help, -?           Display this (help) message"
    print "  -v, --verbose            Increase the debug output level by one"
    print "  -V, --version            Display the CiscoSMFlash version number and exit"


def parse_args(opts):
    """
       Parse the arguments and options and handle them accordingly
    """
    global params, ipAddr, cleanExit

    for o, a in opts:
        if o in ("-?", "-h", "--help"):
            usage()
            return cleanExit
        elif o in ("-v", "--verbose"):
            params.debugLevel += 1
        elif o in ("-V", "--version"):
            print ComDefs.VERSION
            return cleanExit
        elif o in ("-i", "--ipaddress"):
            ipAddr = a
        elif o in ("-s", "--tftpserver"):
            params.tftpIPAddr = a
        elif o in ("-n", "--imagename"):
            params.io1Filename1 = a
        elif o in ("-u", "--username"):
            params.mmUserName = a
        elif o in ("-p", "--password"):
            params.mmPassword = a
        else:
            usage()
            return cleanExit

    return ComDefs.SUCCESS

def verify_args():
    """
       Verify that the required arguments have been supplied.
    """

    global params, ipAddr, cleanExit

    if ipAddr == "":
        print "The hostname or dotted IP address of the I/O Module was not specified."
        usage()
        return cleanExit
    if params.tftpIPAddr == "":
        print "The dotted IP address of the TFTP server was not specified."
        usage()
        return cleanExit
    if params.io1Filename1 == "":
        print "The path and name of the I/O Module firmware image file on the TFTP"
        print "server was not specified."
        usage()
        return cleanExit

    return ComDefs.SUCCESS
    
def flash_switch(mmIPAddr, ioBay=1):
    """ Flash the PM switch via interaction with the telnet interface. """
    global logger, standalone, params, lastError
    
    logger.log("Updating Pass-thru Switch Module firmware.", standalone)

    # Get Variables from params class
    if ioBay == 1:
        filename = params.io1Filename1
    elif ioBay == 2:
        filename = params.io2Filename1
    elif ioBay == 3:
        filename = params.io3Filename1
    elif ioBay == 4:
        filename = params.io4Filename1
     
    userName = params.mmUserName
    password = params.mmPassword

    if filename == "":
        logger.log("Error! Firmware image filename was not supplied.")
        lastError = ComDefs.IO_INVALID_IMAGE_fILE
        return lastError

    
    # Open telnet session to the MM
    telnet = telnetlib.Telnet()
    try:
        telnet.open(mmIPAddr)
        logger.log("Connecting to Management Module to update Pass-thru Switch Module")
    except socket.error:
        logger.log("Socket error! Connection refused or timed out.")
        lastError = ComDefs.TN_SOCKET_ERR
        return lastError
    except KeyboardInterrupt:
        sys.exit(0)
        
    # look for username: and enter credentials
    reList = []
    reList.append(re.compile('.*username:.*'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    
    if index == -1:
        logger.log("Error! Telnet timeout.", standalone)
        telnet.close()
        lastError == ComDefs.GENERAL_UPDATE_ERROR
        return lastError
        
    logger.log("Connected to Management Module")
    telnet.write('%s\r' % userName)
    logger.log("Sending Username and Password.")
    # credentials are sent, continue to the next screen
    reList = []
    reList.append(re.compile('.*password:.*'))
    reList.append(re.compile('.*Invalid login!.*'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    if index == 1:
        logger.log("Error! Incorrect username (%s)." % userName, standalone)
        telnet.close()
        lastError = ComDefs.TN_INVALID_LOGIN
        return lastError
    elif index == -1:
        logger.log("Error! Unhandled timeout.", standalone)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
        
    telnet.write('%s\r' % password)
    reList = []
    reList.append(re.compile('.*system>.*'))
    reList.append(re.compile('.*Invalid login!.*'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)

    if index == 1:
        print txt
        logger.log("Error! Password not accepted.", standalone)
        telnet.close()
        lastError = ComDefs.TN_INVALID_LOGIN
        return lastError
    if index == -1:
        telnet.log("Error! Unhandled telnet timeout.", standalone)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
        
    logger.log("Username and Password accepted.")
    
    # credentials are accepted, continue to firmware upgrade
    # enter TFTP server IP and update image filename
    
    # point MM to switch that we want to update
    switchPrompt = "system:switch[%s]" % ioBay
    telnet.write('env -T %s\r' % switchPrompt)
    reList = []
    reList.append(re.compile('.*OK.*'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT)
    
    if index == -1:
        print txt
        logger.log("Error! Unhandled timeout.", standalone)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    
    telnet.write('update -i %s -l %s\r' % (params.tftpIPAddr, filename))
    reList = []
    reList.append(re.compile('.*OK.*'))
    reList.append(re.compile('.*TFTP Error. File not found.*'))
    reList.append(re.compile('.*TFTP Error -100.*'))
    reList.append(re.compile('.*TFTP Error. Connection failure.*'))
    reList.append(re.compile('.*Error.*'))
    (index,mo,txt) = telnet.expect(reList, ComDefs.TN_TIMEOUT_LONG)
    
    if index == 1:
        logger.log("Error!  Update file not found on TFTP server.", standalone)
        telnet.close()
        lastError = ComDefs.IO_FIRMWARE_IMAGE_NOT_FOUND
        return lastError
    if index == 2:
        logger.log("Error! Update file on TFTP server has wrong permissions.", standalone)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
    if index == 3:
        logger.log("Error! MM was unable to connect to the TFTP server in order to perform the update.", standalone)
        telnet.close()
        lastError = ComDefs.TF_SERVER_NOT_FOUND
        return lastError
    if index == -1:
        logger.log("An unknown timeout has occurred.", standalone)
        telnet.close()
        lastError = ComDefs.GENERAL_UPDATE_ERROR
        return lastError
        
    telnet.write("exit\r")
    telnet.close()
    lastError = ComDefs.SUCCESS
    return lastError

def main():
    """ Main function used to run this script in standalone mode"""

    # Global Variables
    global params, logger, ipAddr, standalone, lastError

    params = ComDefs.parameters()
    standalone = ComDefs.TRUE

    # Parse command line options with getopt()
    try:
        opts, args = getopt.getopt(sys.argv[1:], "?hvVfi:s:n:u:p:",
             ["?", "help", "verbose", "version", "forceUpdate", 
              "ipaddress=", "tftpserver=", "imagename=", "username=", "password="])

    except getopt.GetoptError:
        # Print usage information and exit
        usage()
        sys.exit(0)
    
    lastError = parse_args(opts)
    if lastError != ComDefs.SUCCESS:
        return lastError

    lastError = verify_args()
    if lastError != ComDefs.SUCCESS:
            return lastError
        
    logger = Logger(params)
    
    flash_switch(ipAddr)
        
# This get's invoked if they run this script directly
if __name__ == '__main__':
    main()
    
