#! /usr/bin/env python
###############################################################################
#
#  Script name                  : MMFlash.py
#
#  Dependencies                 : ComDefs.py
#                                 MMComunications.py
#                    
#  Objective                    : Script used to login to the management port (6090)
#                                 of the MM and run dot commands to flash the firmware
#                                 of the MM
#                    
#  Written by                   : Mike Nolterieke
#                                 mikenol@us.ibm.com
#                     
#  Change History               : who -   when   - what
#                                 MHN - 07/05/04 - Initial revision.
#                                 RKC - 07/30/04 - use getopt instead of optparse
#                                 RKC - 07/30/04 - added logging
#                     
###############################################################################
"""
   Script used to login to the management port (6090) of the MM and run dot commands
   to flash the firmware of the MM
"""
try:
    import sys, time, getopt
    sys.path.append("..")
    from Common import ComDefs
    from Common.MMCommunications import MMCom
    from Common.Logger import Logger
except KeyboardInterrupt:
    sys.exit(1)

# The last error that occurred
lastError           = ComDefs.SUCCESS

# The MMCom class
mmCom               = ""

logger = ""
isTest = ComDefs.FALSE
standalone = ComDefs.FALSE
cleanExit = ComDefs.TRUE

# The MM firmware packets and what they are
MM_FIRMWARE_PACKETS = {"Boot ROM":"CNETBRUS.PKT", "Main Application":"CNETMNUS.PKT",
                       "Remote Control":"CNETRGUS.PKT"}
#MM_FIRMWARE_PACKETS = {"Boot ROM":"cnetbrus.pkt", "Main Application":"cnetmnus.pkt",
#                       "Remote Control":"cnetrgus.pkt"}

def connectToMM(params):
    """ Connect to the MM management port (6090) and login
    """
    # Use global variable
    global mmCom
    return (mmCom.open(ComDefs.TRUE, params.mmIPAddress, 6090, 30, params.mmUserName, params.mmPassword))

def updateFirmware(pktFile):
    """ Update the firmware of the MM with the specified packet file
    Return Codes:
        ComDefs.SUCCESS
        ComDefs.MM_PACKET_FILE_NOT_FOUND
        ComDefs.MM_INVALID_PACKET_FILE
        ComDefs.MM_INVALID_PACKET_FILE_HEADER
        Any return code possible from the MMCom.sendPacket function
    """
    global mmCom, lastError
    
    # The packet header format
    pktHeader = '\x16\x02\x00\x03'

    # The default return code
    rc = ComDefs.SUCCESS

    # Try to open the packet file
    try:
        packet = open(pktFile, 'rb')
    # If there was an error opening the file
    except:
        return ComDefs.MM_PACKET_FILE_NOT_FOUND

    j = 0
    while ComDefs.TRUE:
        # Making it display some status while sending the packets
        j += 1
        # We were printing too many .'s
        # Only print every 10th dot
        # Only do this if we're verbose
#        if isTest == ComDefs.TRUE:
#            if ((j % 10) == 0):
#                sys.stdout.write(".")
#                sys.stdout.flush()
                
        # Read SLIM packet header
        header = packet.read(6)

        # If the length of the header is 0, we are done
        if (len(header) == 0):
            break

        # If the length of the header is less than 6, the header
        # is invalid
        if (len(header) < 6):
            rc = ComDefs.MM_INVALID_PACKET_FILE_HEADER
            break

        # If the packet file is invalid
        if (header[:4] != pktHeader):
            rc = ComDefs.MM_INVALID_PACKET_FILE
            break

        # Get the slim packet length
        slimPktLen = (ord(header[4]) * 256) + ord(header[5])

        # Pad all odd length packets with a null byte
        if ((slimPktLen % 2) == 1):
            slimPktLen += 1        

        pkt = packet.read(slimPktLen)

        tail = packet.read(3)
        # If the end of the packet is invalid
        if (tail[2] != '\xC0'):
            rc = ComDefs.MM_INVALID_PACKET_FILE
            break

        # The packet header actually contains the proper dot command
        # to send (8.3.1, 8.3.2, 8.3.3, 8.3.4, etc...) so we do not need to
        # specify the dot command
        # Simply send the entire packet
        rc = mmCom.sendPacket(pkt)
        # If there was an error sending the packet file
        if rc != 0:
            break

    # Close the file
    packet.close()
    return rc

def getUpdateStatus():
    """ Get the current status of the firmware update
    Returns:
        The response from the MM that is the status of the firmware update
        or an empty string if there was an error
    
        The first byte of the response is the status and the second byte of the
        response is the percentage complete

    If an empty string is returned, the error code will be stored in the lastError
    global variable
    """
    global mmCom, lastError
    
    flashStatus = ""

    # Send the 8.3.5 command to the MM to check the update status
    lastError = mmCom.sendCmd('R', '\x08\x03\x05', '')
    # If there was an error 
    if lastError != 0:
        return flashStatus
    # Get the response from the command
    flashStatus = mmCom.getResponse()
    # Reset the lastError variable and return the status
    lastError = ComDefs.SUCCESS
    return flashStatus

def rebootMM(doFailOver=ComDefs.FALSE):
    """ Reboot the MM
    Specify ComDefs.TRUE for doFailOVer if you want the MM to fail over to the redundant MM
    
    Return Codes:
        ComDefs.SUCCESS
        Any return code possible from the MMCom.sendCmd function
    """
    global mmCom
    rc = ComDefs.SUCCESS
    if doFailOver:
        # Send the reboot command and let it fail over if there is a redundant MM
        rc = mmCom.sendCmd('W', '\x07\x04', '')
    else:
        # Send the reboot command and do not let it fail over if there is a redundant MM
        rc = mmCom.sendCmd('W', '\x07\x04\x01', '')
    # Sleep for 20 seconds to let the MM come back to life as per the spec
    time.sleep(20)
    return rc


def usage():
    '''
        Display the Usage Information to the screen
    '''

    print "Usage: MMFlash.py [options]"
    print "\nOptions:"
    print "  -i, --ipaddress=IPADDR   The hostname or dotted IP address of the Management"
    print "                           Module"
    print "  -u, --username=USERNAME  An administrator user name used to log into the"
    print "                           Management Module"
    print "  -p, --password=PASSWORD  The password of the administrator user name used to"
    print "                           log into the Management Module"
    print "  -h, --help, -?           Display this (help) message"
    print "  -v, --verbose            Increase the debug output level by one"
    print "  -V, --version            Display the MMFlash version number and exit"


def parse_args(opts, params):
    """
       Parse the arguments and options and handle them accordingly
    """

    global isTest, cleanExit

    for o, a in opts:
        if o in ("-?", "-h", "--help"):
            usage()
            return cleanExit
        elif o in ("-v", "--verbose"):
            params.debugLevel += 1
        elif o in ("-V", "--version"):
            print ComDefs.VERSION
            return cleanExit
        # Hidden test parameter, specify -t to use our known ip address, user name,
        # and passwords to test this script
        elif o in ("-t", "--test"):
            isTest = ComDefs.TRUE
        elif o in ("-i", "--ipaddress"):
            params.mmIPAddress = a
        elif o in ("-u", "--username"):
            params.mmUserName = a
        elif o in ("-p", "--password"):
            params.mmPassword = a
        else:
            usage()
            return cleanExit

    return ComDefs.SUCCESS


def verify_args(params):
    """
       Verify that the required arguments have been supplied.
    """

    global cleanExit

    if params.mmIPAddress == "":
        print "The hostname or dotted IP address of the Management Module was not specified."
        usage()
        return cleanExit

    return ComDefs.SUCCESS


def main():
    ''' Main function used to run this script in standalone mode'''
    # Define using a global variable
    global mmCom, lastError, logger

    # Instanciate the class
    params = ComDefs.parameters()
    standalone = ComDefs.TRUE

    # Parse command line options with getopt()
    try:
        opts, args = getopt.getopt(sys.argv[1:], "?htvVi:u:p:",
             ["?", "help", "test", "verbose", "version", "ipaddress=", "username=", "password="])

    except getopt.GetoptError:
        # Print usage information and exit
        usage()
        sys.exit(2)

    lastError = parse_args(opts, params)
    if lastError != ComDefs.SUCCESS:
        return lastError

    # If we are in test mode (-t parameter), use our defaults and ignore the rest
    # of the parameters
    if isTest:
        # Populate the class
        params.mmIPAddress     = ""
        params.mmUserName   = ""
        params.mmPassword   = ""
    else:
        lastError = verify_args(params)
        if lastError != ComDefs.SUCCESS:
            return lastError
        
    logger = Logger(params)
    # See if we are in debug mode
    if params.debugLevel == 0:
        # Instanciate the class and leave debug off
        mmCom = MMCom()
    else:
        # Instanciate the class and turn debug on
        mmCom = MMCom(ComDefs.TRUE)
    mmCom.logger = logger
    
    # Get the MM info using the 6090 management port
    # Connect to the MM port 6090
    logger.log("Flashing Management Module", standalone)
    logger.log("Opening connection and logging into the MM")
    if connectToMM(params) != ComDefs.SUCCESS:
        logger.log("There was an error connecting or logging into the MM!", ComDefs.TRUE)
        sys.exit(1)
    else:
        logger.log("Successfully connected and logged into the MM!")

    # Did all of the flashes work?
    flashSuccess    = ComDefs.TRUE

    # Update the MM with all of the packets
    for prettyName, packetName in MM_FIRMWARE_PACKETS.iteritems():
        logger.log("Sending the %s packet to the MM." % prettyName)
        rc = updateFirmware(packetName)
        if rc != ComDefs.SUCCESS:
            flashSuccess = ComDefs.FALSE
            if rc == ComDefs.MM_PACKET_FILE_NOT_FOUND:
                logger.log("Error flashing the MM %s! The packet file %s was not found!" % (prettyName, packetName), ComDefs.TRUE)
            elif ((rc == ComDefs.MM_INVALID_PACKET_FILE) |
                 (rc == ComDefs.MM_INVALID_PACKET_FILE_HEADER)):
                logger.log("Error flashing the MM %s! The packet file %s is not valid!" % (prettyName, packetName), ComDefs.TRUE)
            else:
                logger.log("There was an unknown error flashing the MM %s!" % prettyName, ComDefs.TRUE)
        else:
            print
            logger.log("Flashing the MM %s." % prettyName, standalone)
            flashNotCompleted = ComDefs.TRUE
            isFirstRun = ComDefs.TRUE
            while flashNotCompleted:
                status = getUpdateStatus()
                if isFirstRun:
                    sys.stdout.write("Flash Percent Complete: ")
                    sys.stdout.write(str(ord(status[1])))
                    sys.stdout.write("%")
                    sys.stdout.flush()
                    isFirstRun = ComDefs.FALSE
                else:
                    if status[0] == chr(255):
                        sys.stdout.write('\b\b\b')
                        sys.stdout.write(str(ord(status[1])))
                        sys.stdout.write("%")
                        sys.stdout.flush()
                    if status[0] != chr(255):
                        flashNotCompleted = ComDefs.FALSE
                        print
                        logger.log("Successfully flashed the MM %s." % prettyName, standalone)
                time.sleep(3)

    # If all of the flashes worked, reboot the MM but do not let it fail over
    if flashSuccess:
        print
        logger.log("All flashes completed successfully!", standalone)
        logger.log("Rebooting the MM.", standalone)
        rebootMM()
    else:
        logger.log("One or more of the flashes did not complete successfully! Cannot reboot the MM!", ComDefs.TRUE)
    # Close the connection to the MM
    mmCom.close()
    sys.exit(lastError)
    
# This get's invoked if they run this script directly
if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        ComFuncs.keyboardISR(logger)
